#ifndef INC_INIFILE_H
#define INC_INIFILE_H
//-----------------------------------------------------------------------------
/*!
\file IniFile.h
\brief
\if english
  Header for File
\endif

\if german
  Header für File
\endif

\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the IniFile module.

IniFile is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General
Public License as published by the Free Software Foundation; either version 3 of
the License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "IniFileTypes.h"
#include "PtlTDVList.h"
#include "IniLineFactory.h"
#include "IniSection.h"
#include "IniKey.h"
#include "string"
//-----------------------------------------------------------------------------
// Forwards
namespace eut
{
  class Blob;
}
//-----------------------------------------------------------------------------
namespace ini
{
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
    #pragma warning( disable : 4251 )
  #endif
  //---------------------------------------------------------------------------
  /*!
  \anchor INI_FILE_EXAMPLES
  \brief
  \if english
    An INI file contains information for programs.
    \details An INI file can contain sections with variables and values and comments.
      The information can be created, read and maintained with ini::File.

    \anchor INI_FILE_USING
    <B>Code snippets</B>

    In the following an INI file "test.ini" is created:
  \endif

  \if german
    Eine INI-Datei beinhaltet Informationen für Programme.
    \details Eine INI-Datei kann Sektionen mit Variablen und Werten sowie Kommentare
      enthalten. Die Informationen können mit ini::File angelegt, gelesen und gepflegt
      werden können.

    \anchor INI_FILE_USING
    <B>Codeschnipsel</B>

    Im folgenden wird eine Ini-Datei "test.ini" angelegt:
  \endif

  \code{.cpp}
  #include "IniFile.h"
  #include "EuException.h"
  #include <iostream>

  int main(int argc, char *argv[])
  {
    int         t_iErr = 0;
    ini::File * t_pIniFile = 0;
    //--------------------------------------------------------------------------
    // New Ini-File "test.ini"
    try
    {
      t_pIniFile = new ini::File( "test.ini", ini::bCreateInifile );
    }
    catch( eut::ErrorNException ERR )
    {
      std::cout << "Error :" << ERR.what() <<  "ErrNo.:" << ERR.error() << std::endl;
      return ERR.error();
    }
    // new sections with new keys
    if ( ! t_iErr ) t_iErr = t_pIniFile->setKeyValue( "[Section 1]", "Key 1.1", "Value 1.1" );
    if ( ! t_iErr ) t_iErr = t_pIniFile->setKeyValue( "[Section 1]", "Key 1.2", "Value 1.2" );
    if ( ! t_iErr ) t_iErr = t_pIniFile->setKeyValue( "[Section 2]", "Key 2.1", "Value 2.1" );
    if ( ! t_iErr ) t_iErr = t_pIniFile->setKeyValue( "[Section 2]", "Key 2.2", "Value 2.2" );
    // a comment
    ini::Line  *  t_pCommentLine = 0;
    try
    {
      t_pCommentLine = new ini::Line( "; comment 1" );
    }
    catch( eut::ErrorNException ERR )
    {
      std::cout << "Error :" << ERR.what() <<  "ErrNo.:" << ERR.error() << std::endl;
      t_iErr = ERR.error();
    }
    if ( ! t_iErr )
    {
      t_iErr = t_pIniFile->insertLine( t_pCommentLine );
    }

    // a lonly key
    if ( ! t_iErr ) t_iErr = t_pIniFile->setKeyValue( "", "Lonly Key 1", "Value 1" );
    // and store the Ini-File
    if ( ! t_iErr ) t_iErr = t_pIniFile->store();
    delete t_pIniFile; t_pIniFile = 0;
  }
  \endcode

  \if english
    The INI file "test.ini" has the following contents:
  \endif

  \if german
    Die INI-Datei "test.ini" hat folgenden Inhalt:
  \endif

  \code{.unparsed}
  [Section 1]
  Key 1.1=Value 1.1
  Key 1.2=Value 1.2
  [Section 2]
  Key 2.1=Value 2.1
  Key 2.2=Value 2.2
  ; comment 1
  Lonly Key 1=Value 1
  \endcode

  \if english
    In the INI file "test.ini" the key "Key 1.2" is fetched from the section
    "[Section 1]", displayed and changed to "new Value":
  \endif

  \if german
    In der INI-Datei "test.ini" wird der Schlüssel "Key 1.2" aus der Sektion
    "[Section 1]" geholt, angezeigt und auf "new Value" geändert:
  \endif

  \code{.cpp}
  #include "IniFile.h"
  #include "EuException.h"
  #include <iostream>

  int main(int argc, char *argv[])
  {
    int         t_iErr = 0;
    ini::File * t_pIniFile = 0;
    //--------------------------------------------------------------------------
    // show and change a key in test.ini
    try
    {
      t_pIniFile = new ini::File( "test.ini", ini::bOpenInifile );
    }
    catch( eut::ErrorNException ERR )
    {
      std::cout << "Error :" << ERR.what() <<  "ErrNo.:" << ERR.error() << std::endl;
      return ERR.error();
    }
    // get the value from "Section 1", "Key 1.2"
    std::string t_sValue,
                t_sSection2( "[Section 1]" ),
                t_sKey( "Key 1.2" );

    t_iErr = t_pIniFile->getKeyValue( t_sValue, & t_sSection2, & t_sKey );
    if ( ! t_iErr ) std::cout << "The value from 'Section 1', 'Key 1.2' is: " << t_sValue << std::endl;
    if ( ! t_iErr ) t_iErr = t_pIniFile->setKeyValue( "[Section 1]", "Key 1.2", "new Value" );
    if ( ! t_iErr ) t_iErr = t_pIniFile->store();
    delete t_pIniFile; t_pIniFile = 0;
  }
  \endcode

  \if english
    The INI file "test.ini" now has the following content:
  \endif

  \if german
    Die INI-Datei "test.ini" hat jetzt folgenden Inhalt:
  \endif

  \code{.unparsed}
  [Section 1]
  Key 1.1=Value 1.1
  Key 1.2=new Value
  [Section 2]
  Key 2.1=Value 2.1
  Key 2.2=Value 2.2
  ; comment 1
  Lonly Key 1=Value 1
  \endcode

  \if english
    All entries can be identified...
  \endif

  \if german
    Es können alle Einträge ermittelt werden...
  \endif

  \code{.cpp}
  #include "IniFile.h"
  #include "EuException.h"
  #include <iostream>

  int main(int argc, char *argv[])
  {
    int         t_iErr = 0;
    ini::File * t_pIniFile = 0;
    //--------------------------------------------------------------------------
    // show complete test.ini
    try
    {
      t_pIniFile = new ini::File( "test.ini", ini::bOpenInifile );
    }
    catch( eut::ErrorNException ERR )
    {
      std::cout << "Error :" << ERR.what() <<  "ErrNo.:" << ERR.error() << std::endl;
      return ERR.error();
    }
    ini::Line  *  t_pShowLine = 0;

    std::cout << "############# " << t_pIniFile->getFileName() << " #############" << std::endl;
    t_iErr = t_pIniFile->getLine( t_pShowLine, 0, ptl::START );
    while ( t_pShowLine )
    {
      std::cout << t_pShowLine->getText() << std::endl;
      // if section, show sectionlines
      if ( t_pShowLine->getType() == ini::LINE_TYPE::IniSECTION )
      {
        ini::Section  * t_pCastSectionLine = dynamic_cast<ini::Section*>(t_pShowLine);
        ini::Line     * t_pShowSectionLine = 0;

        t_iErr = t_pCastSectionLine->getSectionLine( t_pShowSectionLine, 0, ptl::START );
        while ( t_pShowSectionLine )
        {
          std::cout << t_pShowSectionLine->getText() << std::endl;
          t_iErr = t_pCastSectionLine->getSectionLine( t_pShowSectionLine, 1, ptl::CURRENT );
        }
        t_iErr = ( t_iErr == eut::ERR_RANGE ? 0 : t_iErr );
      }
      t_iErr = t_pIniFile->getLine( t_pShowLine, 1, ptl::CURRENT );
    }
    t_iErr = ( t_iErr == ptl::RANGE ? 0 : t_iErr );
    std::cout << "####################################" << std::endl;
    std::cout << "ErrNo.:" << t_iErr << std::endl;
    delete t_pIniFile; t_pIniFile = 0;
  }
  \endcode

  \if english
    ... and results in the following output:
  \endif

  \if german
    ... und ergibt folgende Anzeige:
  \endif

  \code{.unparsed}
  ############# test.ini #############
  [Section 1]
  Key 1.1=Value 1.1
  Key 1.2=new Value
  [Section 2]
  Key 2.1=Value 2.1
  Key 2.2=Value 2.2
  ; comment 1
  Lonly Key 1=Value 1
  ####################################
  ErrNo.:0
  \endcode
  \author Helmut Jakoby
  \copyright © 1995 Helmut Jakoby
  */
  class __ini_file_export_dll File
  {
    private:
      //============== Attribute ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        The name of the INI file is stored in this attribute.
      \endif
    
      \if german
        In diesem Attribut wird die Bezeichnung der INI-Datei abgelegt.
      \endif
      */
      std::string m_sFileName;
      //-----------------------------------------------------------------------
      /*!
      \if english
        List of entries in this INI file.
      \endif
    
      \if german
        Liste der Einträge in dieser INI-Datei.
      \endif
      */
      ptl::TDVList<Line> m_LineList;
      //-----------------------------------------------------------------------
      /*!
      \if english
        The LineFactory returns an INI file entry according to type.
      \endif
    
      \if german
        Die LineFactory liefert einen INI_Datei Eintrag typgerecht.
      \endif
      */
      LineFactory m_LineFactory;
      //-----------------------------------------------------------------------

    public:
      //============== Konstruktoren ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor with parameter passing.

        \param[in] rsFullFileName The complete file name such as: "c:\windows\system.ini"
        \param[in] openMode If openMode is ini::bCreateInifile, then an attempt is made
          to generate a new INI-file. <br>
          If openMode is ini::bOpenInifile, then an attempt is made to read in the
          INI-file with the specified name. 
        \exception eut::ErrorNException An exception is thrown if an error
          occurs.
      \endif
    
      \if german
        Konstruktor mit Parameterübergabe.

        \param[in] rsFullFileName Die vollständige Dateibezeichnung wie z.B.: "c:\windows\system.ini".
        \param[in] openMode Wenn openMode auf ini::bCreateInifile, dann wird versucht, eine neue INI-Datei zu generieren. <br>
          Wenn openMode auf ini::bOpenInifile, dann wird versucht, die INI-Datei mit der übergebenen Bezeichnung einzulesen.
        \exception eut::ErrorNException Bei einem Fehler wird eine Exception
          geworfen.
      \endif
      */
      File( const std::string & rsFullFileName, TdOpenInifileMode openMode );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Default constructor.
      \endif
    
      \if german
        Standard-Konstruktor.
      \endif
      */
      File();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor, memory is released.
      \endif
    
      \if german
        Destruktor, Speicher wird freigegeben.
      \endif
      */
      virtual ~File();
      //-----------------------------------------------------------------------

    private:
      //-----------------------------------------------------------------------
      /*!
      \if english
        The copy constructor is not available.
      \endif
    
      \if german
        Der Copy-Konstruktor steht nicht zur Verfügung.
      \endif
      */
      File( const File & );
      //-----------------------------------------------------------------------

    public:
      //============== Datei-Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        The INI file from #m_sFileName is saved.
        \return A return < 0 indicates an error.
      \endif
    
      \if german
        Es wird die INI-Datei aus #m_sFileName gespeichert.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int store();
      //-----------------------------------------------------------------------
      /*!
      \if english
        The INI file from #m_sFileName is read in.
        \return A return < 0 indicates an error.
        \warning All possible changes are discarded.
      \endif
    
      \if german
        Es wird die INI-Datei aus #m_sFileName eingelesen.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
        \warning Alle evtl. Änderungen werden verworfen.
      \endif
      */
      int refresh();
      //-----------------------------------------------------------------------

      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sets the name of the INI file.
        \param[in] rsFullFileName The complete file name such as: "c:\windows\system.ini"
      \endif
    
      \if german
        Setzt die Bezeichnung der INI-Datei.
        \param[in] rsFullFileName Die vollständige Dateibezeichnung wie z.B.:
          "c:\windows\system.ini".
      \endif
      */
      void setFullFileName( const std::string & rsFullFileName );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the name of the INI file.
        \return The name.
      \endif
    
      \if german
        Liefert die Bezeichnung der INI-Datei.
        \return Die Bezeichnung.
      \endif
      */
      std::string getFileName();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns an Line from the #m_LineList with respect to the pass parameters.
        \param[in,out] prRetVal passed reference into which the found object  "comes in"
        \param[in] nPosition Offset relative to...
        \param[in] eMode ...Position

        <B>Parameter examples:</B>
        \li Delivers first element from list: lPosition = 0; eMode = ptl::START
        \li Delivers last element from list: lPosition = 0; eMode = ptl::END
        \li Delivers current element from list: lPosition = 0; eMode = ptl::CURRENT_POS
        \li Delivers next element from list: lPosition = 1; eMode = ptl::CURRENT_POS
        \li Returns previous element from list: lPosition = -1; eMode = ptl::CURRENT_POS

        \return A return < 0 indicates an error.
        \attention The returned object (in prRetVal) must not be removed from memory
          by the calling instance!
      \endif
    
      \if german
        Liefert einen Line aus der #m_LineList in Bezug auf die ÜbergabeParameter.
        \param[in,out] prRetVal übergebene Referenz in welche das gefundene Objekt
          "hineinkommt"
        \param[in] nPosition Offset rellativ zur...
        \param[in] eMode ...Position

        <B>Parameterbeispiele:</B>
        \li Liefert erstes Element aus Liste: lPosition =  0; eMode = ptl::START
        \li Liefert letztes Element aus Liste: lPosition =  0; eMode = ptl::END
        \li Liefert aktuelles Element aus Liste: lPosition =  0; eMode = ptl::CURRENT_POS
        \li Liefert nächstes Element aus Liste: lPosition =  1; eMode = ptl::CURRENT_POS
        \li Liefert vorgeriges Element aus Liste: lPosition = -1; eMode = ptl::CURRENT_POS

        \return Eine Rückgabe < 0 zeigt einen Fehler an.
        \attention Das gelieferte Objekt (in prRetVal) darf NICHT von der aufrufenden
          Instanz aus dem Speicher entfernt werden!
      \endif
      */
      int getLine( Line *& prRetVal, std::size_t nPosition, ptl::SeekMode eMode );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Inserts the transferred entry into the #m_LineList.
        \param[in] pIniLine The object of the class Line to be inserted
        \param[in] eInsertPosition Shows where the data object is inserted in the
          list is to be inserted. Possible parameters are <B>ptl::InsertHEAD</B>
          (on Start), <B>ptl::InsertLAST</B> (at the end) or <B>ptl::InsertCURRENT</B>
          (at the current element).
          \param[in] eBeforeBehind Specifies whether the new element is before or
          after the, element specified in the eInsertPosition parameter. Possible
          Parameters are <B>BEHIND</B> and <B>BEFORE</B>
        \return A return value < 0 indicates an error.
        \attention ini::File assumes responsibility for the Line transferred in
          pIniLine. This means that the calling instance must not remove the passed
          object from memory!
      \endif
    
      \if german
        Fügt den übergebenen Eintrag in die #m_LineList ein.
        \param[in] pIniLine Das einzufügende Objekt der Klasse Line
        \param[in] eInsertPosition Zeigt an, wohin das Datenobjekt in die Liste
          eingefügt werden soll. Mögliche Parameter sind <B>ptl::InsertHEAD</B> (am
          Anfang), <B>ptl::InsertLAST</B> (am Ende) oder <B>ptl::InsertCURRENT</B> (am
          aktuellen Element).
          \param[in] eBeforeBehind Gibt an, ob das neue Element vor oder hinter das,
          im Parameter eInsertPosition angegebene Element, gesetzt wird. Mögliche
          Parameter sind <B>BEHIND</B> und <B>BEFORE</B>
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
        \attention ini::File übernimmt die Verantwortung für die in pIniLine übergebene
          Line. Das bedeutet, die aufrufende Instanz darf das übergebene Objekt nicht
          aus dem Speicher entfernen.
      \endif
      */
      int insertLine( Line * pIniLine, 
                      ptl::InsertPosition eInsertPosition = ptl::InsertCURRENT, 
                      ptl::VH eBeforeBehind = ptl::BEHIND );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Removes the entry from the #m_LineList.

        If a subsequent element exists, it becomes the current element, otherwise
        the previous element if it exists. The destructor is called from the removed
        entry.
        \param[in] eFirstLastCurrent The following values are possible: <B>ptl::FIRST_POS</B>,
          <B>ptl::LAST_POS</B>, (default) <B>ptl::CURRENT_POS</B>, <B>ptl::NEXT_POS</B> and
          <B>ptl::PREVIOUS_POS</B>.
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Entfernt den Eintrag aus der #m_LineList.

        Wenn ein nachfolgendes Element vorhanden ist, wird dieses zum aktuellen Element,
        ansonsten das vorherige Element, wenn dieses vorhanden ist. Vom entfernten
        Eintrag wird der Destruktor aufgerufen.
        \param[in] eFirstLastCurrent Folgende Werte sind möglich: <B>ptl::FIRST_POS</B>,
          <B>ptl::LAST_POS</B>, (default) <B>ptl::CURRENT_POS</B>, <B>ptl::NEXT_POS</B> und
          <B>ptl::PREVIOUS_POS</B>.
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
      \endif
      */
      int removeLine( ptl::ElementPosition eFirstLastCurrent = ptl::CURRENT_POS );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns a section or a key.
        \param[in,out] prRetVal If found, depending on the other parameters a
          pointer to an object of type Section or Key.
        \param[in] psSection
          \li If set and psKey not set, the Section is searched for.
          \li If set and psKey is also set, the Key from the psSection wanted.
          \li If not set and psKey is set, the Key without section is searched.
        \param[in] psKey See psSection.
        \return A return value < 0 indicates an error.
        \attention The object returned in prRetVal must not be removed from memory
          by the calling instance!


      \endif
    
      \if german
        Liefert eine Sektion oder einen Key.
        \param[in,out] prRetVal Wenn gefunden, in Abhängigkeit der anderen Parameter
          ein Zeiger auf ein Objekt vom Typ Section oder Key.
        \param[in] psSection
          \li Wenn gesetzt und psKey nicht gesetzt, wird die Section gesucht.
          \li Wenn gesetzt und psKey auch gesetzt, wird der Key aus der psSection
            gesucht.
          \li Wenn nicht gesetzt und psKey gesetzt, wird der Key ohne Sektion gesucht.
        \param[in] psKey Siehe psSection.
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
        \attention Das gelieferte Objekt in prRetVal darf NICHT von der aufrufenden
          Instanz aus dem Speicher entfernt werden!
      \endif
      */
      int getLine( Line *& prRetVal, std::string * psSection, std::string * psKey = 0 );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns a KeyValue.
        \param[in,out] rsValue The found KeyValue as std::string.
        \param[in] psSection
          \li If set and psKey not set -> Error.
          \li If set and psKey is also set, the Key from the psSection wanted.
          \li If not set and psKey is set, the Key without section is searched.
        \param[in] psKey See psSection.

        If there should be the following entry in an INI file:
        \code{.unparsed}
        [386enh]
        woafont=app850.fon <- Variable with value
        \endcode

        returns this function when calling:

        \code{.cpp}
        int t_iErr = pMyIniFile->getKeyValue( sValue, "[386nh]", "woafont");
        \endcode
        in rsValue "app850.fon" back.
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Liefert ein KeyValue.
        \param[in,out] rsValue Der gefundene KeyValue als std::string.
        \param[in] psSection
          \li Wenn gesetzt und psKey nicht gesetzt -> Fehler.
          \li Wenn gesetzt und psKey auch gesetzt, wird der Key aus der psSection
            gesucht.
          \li Wenn nicht gesetzt und psKey gesetzt, wird der Key ohne Sektion gesucht.
        \param[in] psKey Siehe psSection.

        Wenn es folgenden Eintrag in einer INI-Datei geben sollte:
        \code{.unparsed}
        [386enh]
        woafont=app850.fon      <- Variable mit Wert
        \endcode

        liefert diese Funktion beim Aufruf von:

        \code{.cpp}
        int t_iErr = pMyIniDatei->getKeyValue( sValue, "[386nh]", "woafont");
        \endcode
        in rsValue "app850.fon" zurück.
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
       \endif
     */
      int getKeyValue( std::string & rsValue, std::string * psSection, std::string * psKey );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sets an entry
        \param[in] rsSection The section either with '[' and ']' or without these
          two characters.
        \param[in] rsKey The entry
        \param[in] rsValue The value

        Sets an entry like this:
        - If section is passed, it is searched for. If it does not exist, the
          section is created. If no section is passed, the INI file is the section.
        - If entry and value are passed, the variable from the entry is searched
          for in the section. If it is found, the value is set, otherwise the
          entire variable with value is inserted.

        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Setzt einen Eintrag
        \param[in] rsSection Die Sektion entweder mit '[' und ']' oder ohne diesen
        beiden Zeichen.
        \param[in] rsKey Der Eintrag
        \param[in] rsValue Der Wert

        Setzt einen Eintrag folgendermassen:
        - Wenn Sektion übergeben wird, wird diese gesucht. Wenn nicht vorhanden,
          wird die Sektion angelegt. Wenn keine Sektion übergeben wird, ist die
          INI-Datei die Sektion.
        - Wenn Eintrag und Wert übergeben wird, wird in der Sektion die Variable
          aus dem Eintrag gesucht. Wenn diese gefundenwird, wird der Wert gesetzt,
          ansonsten wird die gesamte Variable mit Wert eingefügt.

        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
      \endif
      */
      int setKeyValue(  const std::string & rsSection,
                        const std::string & rsKey,
                        const std::string & rsValue );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Removes an entry
        \param[in] rsSection Section
        \param[in] rsKey entry

        Removes an entry as follows:
        - If section is passed, this is searched for. If present and entry is not
          set, the section is removed from the INI file.
        - If entry is passed, the variable from the entry is searched for in the
          section or INI file. If found, the variable is removed from the section or INI file.

        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Entfernt einen Eintrag
        \param[in] rsSection Sektion
        \param[in] rsKey Eintrag

        Entfernt einen Eintrag folgendermassen:
        - Wenn Sektion übergeben, wird diese gesucht. Wenn vorhanden und Eintrag
          ist nicht gesetzt, wird diese aus der INI-Datei entfernt.
        - Wenn Eintrag übergeben, dann wird in der Sektion bzw. INI-Datei die
          Variable aus dem Eintrag gesucht. Wenn gefunden wird die Variable aus der
          Sektion bzw. INI-Datei entfernt.

        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
      \endif
      */
      int removeKey( const std::string & rsSection, const std::string & rsKey );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns all entries of the file in one blob.
        \param[in,out] prBlob The File as blob.
        \return A return value < 0 indicates an error.
        \attention The calling instance must remove the delivered blob from memory.
      \endif
    
      \if german
        Liefert alle Einträge der Datei in einem Blob.
        \param[in,out] prBlob Das File als Blob.
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
        \attention Die aufrufende Instanz muss das gelieferten Blob aus den
          Speicher entfernen.
      \endif
      */
      int getAsBlob( eut::Blob *& prBlob );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Creates the individual entries in the INI file from the blob passed.
        \param[in] pBlob The blob with the contents of an INI file. The blob is
          evaluated and all entries are created.
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Erstellt aus übergebenen Blob die einzelnen Einträge in der INI-Datei.
        \param[in] pBlob Das Blob mit dem Inhalt einer Ini-Datei. Das Blob wird
          ausgewertet und alle Einträge erstellt.
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
      \endif
      */
      int makeLines( eut::Blob * pBlob );
      //-----------------------------------------------------------------------

    private:
      //============== Operatoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        Assignment operator is not available.
      \endif
    
      \if german
        Zuweisungsoperator steht nicht zur Verfügung.
      \endif
      */
      File & operator= ( const File & );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
    #pragma warning( default : 4251 )
  #endif
  //---------------------------------------------------------------------------
} // namespace ini
//-----------------------------------------------------------------------------
#endif
