#ifndef INC_INI_FILE_TYPES_H
#define INC_INI_FILE_TYPES_H
//-----------------------------------------------------------------------------
/*!
\file IniFileTypes.h
\brief
\if english
  For each library, here 'IniFile' there is a type definition file.

  This file is usually included by every header from the library.
\endif

\if german
  Für jede Bibliothek, hier 'IniFile' gibt es eine Typen-Datei.

  Diese Datei wird in der Regel von jeder Header aus der Bibliothek eingebunden.
\endif

\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the IniFile module.

IniFile is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General
Public License as published by the Free Software Foundation; either version 3 of
the License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include <string>
#include "IniFileErrors.h"
#include "EuDef_NULL_PTR.h"
//-----------------------------------------------------------------------------
// DLL-Export und -Import definition
#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
# ifdef __INI_FILE_EXPORT_DLL
#   define __ini_file_export_dll  __declspec(dllexport)
#   if defined(_MSC_VER)
#     pragma message ( "Benutze Export defines fuer __ini_file_export_dll" )
#   endif
# else
#   define __ini_file_export_dll  //__declspec(dllimport)
#   if defined(_MSC_VER)
#     pragma message ( "Benutze Import defines fuer __ini_file_export_dll" )
#   endif
# endif
#elif defined(__GNUC__)
# ifdef __INI_FILE_EXPORT_DLL
#   define __ini_file_export_dll
# else
#   define __ini_file_export_dll
# endif
#endif
//=============================================================================
//-----------------------------------------------------------------------------
namespace ini
{
  //---------------------------------------------------------------------------
  /*!
  \if english
    The version. If new, change it here.

    The file names of the DLL or LIB file are composed of the project name + the
    compiler identification + the version of the library.

    For example, for Visual Studio 2015 = "$(ProjectName)$(PlatformToolset)101"
    as the target name; if the compile target is "INIFILE", the name becomes
    "INIFILEv140101.dll" and "INIFILEv140101.lib".
  \endif

  \if german
    Die Version. Wenn neu, hier ändern.

    Die Dateibezeichnungen der DLL- bzw. der LIB-Datei setzt sich aus der
    Projektbezeichnung + der Compilerkennung + der Version der Bibliothek
    zusammmen.

    Z.B. für das Visual Studio 2015 = "$(ProjectName)$(PlatformToolset)101" als Zielname;
    wird die Bezeichnung, wenn das Kompilierziel "INIFILE", zu "INIFILEv140101.dll"
    bzw. "INIFILEv140101.lib"
  \endif
  */
  const std::string g_sIniFileVersion = "1.0.2";
  //---------------------------------------------------------------------------
  /*!
  \if english
    Different operating systems probably have different line separators in
    Files. So here for everything "\n"!
  \endif
  
  \if german
    Verschiedene Beriebssysteme haben wohl unterschiedliche Zeilentrenner in
    Dateien. Also hier für alles "\n"!
  \endif
  */
  const std::string g_sEOL = "\n";
  //---------------------------------------------------------------------------
  /*!
  \if english
    The type of an entry in an ini file is defined by LINE_TYPE.
  \endif

  \if german
    Der Typ eines Eintrags in einem Ini-File ist durch LINE_TYPE definiert.
  \endif
  */
  enum LINE_TYPE
  {
    /*!
    \if english
      Entry not specified
    \endif
    
    \if german
      Eintrag nicht spezifiziert
    \endif
    */
    IniUNDEFINED,
    /*!
    \if english
      Entry is a section -> [Section 1]
    \endif
    
    \if german
      Eintrag ist eine Sektion -> [Section 1]
    \endif
    */
    IniSECTION,
    /*!
    \if english
      Entry is a comment -> start of line ';' or '#'.
    \endif
    
    \if german
      Eintrag ist ein Kommentar -> Zeilenanfang ';' oder '#'
    \endif
    */
    IniCOMMENT,
    /*!
    \if english
      Entry is a variable possibly with value -> woafont=app850.fon
    \endif
    
    \if german
      Eintrag ist eine Variable ggf. mit Wert -> woafont=app850.fon
    \endif
    */
    IniVARIABLE,
    /*!
    \if english
      Eintrag ist eine Leerzeile
    \endif
    
    \if german
      Eintrag ist eine Leerzeile
    \endif
    */
    IniEMPTY_LINE
  };
  //---------------------------------------------------------------------------
  // 
  //---------------------------------------------------------------------------
  /*!
  \if english
    To make the <b>bOpenMode</b> parameter in the constructor of ini::File more readable. 
  \endif

  \if german
    Um den Parameter <b>bOpenMode</b> im Konstruktor von ini::File lesbarer zu machen.
  \endif

  \sa ini::File::File( const std::string & rsFullFileName, TdOpenInifileMode openMode )
  */
  typedef bool TdOpenInifileMode;
  //---------------------------------------------------------------------------
  /*!
  \if english
    Signals the constructor to create a new ini file.
  \endif

  \if german
    Signalisiert dem Konstruktor, das eine neue Ini-Datei erstellt werden soll.
  \endif

  \sa ini::File::File( const std::string & rsFullFileName, TdOpenInifileMode openMode )
  */
  const TdOpenInifileMode bCreateInifile = false;
  //---------------------------------------------------------------------------
  /*!
  \if english
    Signals the constructor that an existing ini file should be read.
  \endif

  \if german
    Signalisiert dem Konstruktor, das eine vorhandene Ini-Datei eingelesen werden
    soll.
  \endif

  \sa ini::File::File( const std::string & rsFullFileName, TdOpenInifileMode openMode )
  */
  const TdOpenInifileMode bOpenInifile = true;
  //---------------------------------------------------------------------------
} // namespace ini
//-----------------------------------------------------------------------------
#endif
