#ifndef INC_INIKEY_H
#define INC_INIKEY_H
//-----------------------------------------------------------------------------
/*!
\file IniKey.h
\brief
\if english
  Header for Key
\endif

\if german
  Header für Key
\endif
\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the IniFile module.

IniFile is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General
Public License as published by the Free Software Foundation; either version 3 of
the License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "IniFileTypes.h"
#include "IniLine.h"
//-----------------------------------------------------------------------------
namespace ini
{
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    A variable with value possibly also in a section like "Pixel=1024".
    Variable and value are separated by a "=".
  \endif

  \if german
    Eine Variable mit Wert eventuell auch in einer Sektion wie z.B.: "Pixel=1024".
    Variable und Wert sind durch ein "=" getrennt.
  \endif

  \author Helmut Jakoby
  \copyright © 1995 Helmut Jakoby
  */
  class __ini_file_export_dll Key : public Line
  {
    private:
      //============== Friends
      //-----------------------------------------------------------------------
      friend class LineFactory;
      //-----------------------------------------------------------------------

    public:
      //============== Konstruktoren ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor with parameter passing.

        \param[in] sCharLine The text of the entry.
      \endif
    
      \if german
        Konstruktor mit Parameterübergabe.

        \param[in] sCharLine Der Text des Eintrags.
      \endif
      */
      Key( const std::string & sCharLine );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor.
      \endif
    
      \if german
        Destruktor.
      \endif
      */
      virtual ~Key ();
      //-----------------------------------------------------------------------

    protected:
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor with parameter passing is not publicly available.

        \param[in] sCharLine The text of the entry.
        \param[in] eLineType The type of the entry, the calling instance must have
          determined the correct type, here #IniVARIABLE.
      \endif
    
      \if german
        Dieser Konstruktor mit Parameterübergabe steht nicht öffendlich zur Verfügung.

        \param[in] sCharLine Der Text des Eintrags.
        \param[in] eLineType Der Typ des Eintrags, die Aufrufende Instanz muss den
          richtigen Typen, hier #IniVARIABLE, ermittelt haben.
      \endif
      */
      Key( const std::string & sCharLine, LINE_TYPE eLineType );
      //-----------------------------------------------------------------------
      /*!
      \if english
        The standard constructor is not publicly available.
      \endif
    
      \if german
        Der Standartkonstruktor steht nicht öffendlich zur Verfügung.
      \endif
      */
      Key();
      //-----------------------------------------------------------------------

    private:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The copy constructor is not available.
      \endif
    
      \if german
        Der Copy-Konstruktor steht nicht zur Verfügung.
      \endif
      */
      Key( const Key & );
      //-----------------------------------------------------------------------

    public:
      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the name of the variable.
        \return The variable name as string <B>without</B> appended "=".
      \endif
    
      \if german
        Liefert die Bezeichnung der Variable.
        \return Die Variablenbezeichnung als String <B>ohne</B> angehängtes "=".
      \endif
      */
      std::string getKeyName() const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sets the name of the variable.
        \param[in] sName The name as string <B>without</B> appended "=".

        \note If the INI entry was previously "Resolution=1024" and the variable
          name was changed to "Pixel", the new INI entry is "Pixel=1024".
      \endif
    
      \if german
        Setzt die Bezeichnung der Variable.
        \param[in] sName Die Bezeichnung als String <B>ohne</B> angehängtes "=".

        \note Wenn der INI-Eintrag vorher "Aufloesung=1024" war und die Bezeichnung
          der Variablen auf "Pixel" geändert wurd, ist der neue INI-Eintrag
          "Pixel=1024".
      \endif
      */
      void setKeyName( const std::string & sName );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the value of the variable.
        \return The value as string <B>without</B> leading "=".
      \endif
    
      \if german
        Liefert den Wert der Variable.
        \return Der Wert als String <B>ohne</B> führendes "=".
      \endif
      */
      std::string getKeyValue() const ;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sets the value of the variable.
        \param[in] sValue The value as string <B>without</B> leading "=".

        \note If the INI entry was previously "Resolution=1024" and the value of
          the variable was changed to "640", the new INI entry is "Resolution=640".
      \endif
    
      \if german
        Setzt den Wert der Variable.
        \param[in] sValue Der Wert als String <B>ohne</B> führendes "=".

        \note Wenn der INI-Eintrag vorher "Aufloesung=1024" war und der Wert der
          Variablen auf "640" geändert wurd, ist der neue INI-Eintrag "Aufloesung=640".
      \endif
      */
      void setKeyValue ( const std::string & sValue );
      //-----------------------------------------------------------------------

    private:
      //============== Operatoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The assignment operator is not available.
      \endif
    
      \if german
        Der Zuweisungsoperator steht nicht zur Verfügung.
      \endif
      */
      Key & operator= ( const Key & );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
} // namespace ini
//-----------------------------------------------------------------------------
#endif
