#ifndef INC_INILINEFACTORY_H
#define INC_INILINEFACTORY_H
//-----------------------------------------------------------------------------
/*!
\file IniLineFactory.h
\brief
\if english
  Header for LineFactory
\endif

\if german
  Header für LineFactory
\endif
\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the IniFile module.

IniFile is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General
Public License as published by the Free Software Foundation; either version 3 of
the License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "IniFileTypes.h"
//-----------------------------------------------------------------------------
namespace ini
{
  //-----------------------------------------------------------------------------
  /*! Forwards */
  class Line;
  //-----------------------------------------------------------------------------
  /*!
  \brief
  \if english
    This class turns lines of an INI file into objects of the types Line,
    Section or Key.
  \endif

  \if german
    Diese Klasse macht aus Zeilen einer INI-Datei Objekte der Typen Line,
    Section oder Key.
  \endif

  \author Helmut Jakoby
  \copyright © 1995 Helmut Jakoby
  */
  class __ini_file_export_dll LineFactory
  {
    public:
      //============== Konstruktoren ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        Standard constructor.
      \endif
    
      \if german
        Standartkonstruktor.
      \endif
      */
      LineFactory();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor.
      \endif
    
      \if german
        Destruktor.
      \endif
      */
      virtual ~LineFactory();
      //-----------------------------------------------------------------------

    private:
      //============== Konstruktoren ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        The copy constructor is not available.
      \endif
    
      \if german
        Der Copy-Konstruktor steht nicht zur Verfügung.
      \endif
      */
      inline LineFactory( const LineFactory & );
      //-----------------------------------------------------------------------

    public:
      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        This method returns a pointer to an Line or subclass of Line in the
        reference 'prIniLine'.
        \param[in,out] prIniLine Pointer to a newly instantiated Line or subclass.
        \param[in] sCharLine The line from an ini file as string. The text line is
          evaluated accordingly.
        \return A return < 0 indicates an error.
        \attention The calling instance must remove a supplied valid Line or
          subclass from memory.
      \endif
    
      \if german
        Diese Methode liefert in übergebene Referenz 'prIniLine' einen Zeiger auf
        einen Line oder Unterklasse von Line.
        \param[in,out] prIniLine Zeiger auf eine neu instanziierte Line oder Unterklasse.
        \param[in] sCharLine Die Zeile aus einer Ini-Datei als String. Die Textzeile wird
          entsprechend ausgewertet.
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
        \attention Die aufrufende Instanz muß eine gelieferte gültige Line oder
          Unterklasse wieder aus dem Speicher entfernen.
      \endif
      */
      int getIniLine( Line *& prIniLine, std::string sCharLine ) const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Removes leading and trailing spaces {0x20} and TABs {0x09} in the passed
        string.
        \param[in,out] rsCharLine Reference to string to be processed.
      \endif
    
      \if german
        Entfernt im übergebenen String führende und angehängten Leerzeichen {0x20}
        und TABs {0x09}.
        \param[in,out] rsCharLine Referenz auf zu bearbeitenden String.
      \endif
      */
      void trim( std::string & rsCharLine ) const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the type of the entry.
        \param[in] sCharLine Line to be analyzed as string.
        \return The determined type of the entry line.
      \endif
    
      \if german
        Liefert den Typ des Eintrags.
        \param[in] sCharLine Zu analysierende Zeile als String.
        \return Der ermittelte Typ der Eintragszeile.
      \endif
      */
      LINE_TYPE getLineType( std::string sCharLine ) const;
      //-----------------------------------------------------------------------

    private:
      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Checks if the transferred entry is an INI comment.
        \param[in] sCharLine Line to be analyzed as string.
        \return Returns true if the passed entry is an INI comment, otherwise false.
      \endif
    
      \if german
        Prüft ob übergebener Eintrag ein INI-Kommentar ist.
        \param[in] sCharLine Zu analysierende Zeile als String.
        \return Liefert true, wenn übergebener Eintrag ein INI-Kommentar ist,
          ansonste false.
      \endif
      */
      bool isComment( std::string sCharLine ) const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Checks whether the transferred entry is an INI section.
        \param[in] sCharLine Line to be analyzed as string.
        \return Returns true if the given entry is an INI section, otherwise false.
      \endif
    
      \if german
        Prüft ob übergebener Eintrag eine INI-Sektion ist.
        \param[in] sCharLine Zu analysierende Zeile als String.
        \return Liefert true, wenn übergebener Eintrag eine INI-Sektion ist,
          ansonste false.
      \endif
      */
      bool isSection( std::string sCharLine ) const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Checks whether the transferred entry is an INI key.
        \param[in] sCharLine Line to be analyzed as string.
        \return Returns true, if passed entry is an INI key, otherwise false.
      \endif
    
      \if german
        Prüft ob übergebener Eintrag ein INI-Key ist.
        \param[in] sCharLine Zu analysierende Zeile als  String.
        \return Liefert true, wenn übergebener Eintrag ein INI-Key ist, ansonste
          false.
      \endif
      */
      bool isKey( std::string sCharLine ) const;
      //-----------------------------------------------------------------------

    private:
      //============== Operatoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The assignment operator is not available.
      \endif
    
      \if german
        Der Zuweisungsoperator steht nicht zur Verfügung.
      \endif
      */
      inline LineFactory& operator=( const LineFactory & );
      //-----------------------------------------------------------------------
  };  // class __ini_file_export_dll LineFactory
  //---------------------------------------------------------------------------
} // namespace ini
//-----------------------------------------------------------------------------
#endif
