#ifndef INC_INISECTION_H
#define INC_INISECTION_H
//-----------------------------------------------------------------------------
/*!
\file IniSection.h
\brief
\if english
  Header for Section
\endif

\if german
  Header für Section
\endif
\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the IniFile module.

IniFile is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General
Public License as published by the Free Software Foundation; either version 3 of
the License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "IniFileTypes.h"
#include "IniLine.h"
#include "PtlTDVList.h"
//-----------------------------------------------------------------------------
/*! Forwards */
namespace eut
{
  class Blob;
}
//-----------------------------------------------------------------------------
namespace ini
{
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von Clients von Klasse 'Typ2' verwendet wird */
    #pragma warning( disable : 4251 )
  #endif
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    A section can only appear once in an INI file, is enclosed in square brackets
    and usually has variables with values.
  \endif

  \if german
    Eine Sektion kann nur einmalig in einer INI-Datei vorkommen, steht in
    eckigen Klammern und hat in der Regel Variablen mit Werten.
  \endif

  \author Helmut Jakoby
  \copyright © 1995 Helmut Jakoby
  */
  class __ini_file_export_dll Section : public Line
  {
    private:
      //============== Friends
      //------------------------------------------------------------------------
      friend class LineFactory;
      friend class File;
      //------------------------------------------------------------------------

      //============== Attribute ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        List of entries in this section.
      \endif
    
      \if german
        Liste der Einträge dieser Sektion.
      \endif
      */
      ptl::TDVList<Line> m_SektionList;
      //-----------------------------------------------------------------------

    public:
      //============== Konstruktoren ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor with parameter passing.

        \param[in] sCharLine The text of the entry.
      \endif
    
      \if german
        Konstruktor mit Parameterübergabe.

        \param[in] sCharLine Der Text des Eintrags.
      \endif
      */
      Section( const std::string & sCharLine );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor.
      \endif
    
      \if german
        Destruktor.
      \endif
      */
      virtual ~Section();
      //-----------------------------------------------------------------------

    protected:
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor with parameter passing is not publicly available.

        \param[in] sCharLine The text of the entry.
        \param[in] eLineType The type of the entry, the calling instance must have
          determined the correct type, here #IniSECTION.
      \endif
    
      \if german
        Dieser Konstruktor mit Parameterübergabe steht nicht öffendlich zur Verfügung.

        \param[in] sCharLine Der Text des Eintrags.
        \param[in] eLineType Der Typ des Eintrags, die Aufrufende Instanz muss den
          richtigen Typen, hier #IniSECTION, ermittelt haben.
      \endif
      */
      Section( const std::string & sCharLine, LINE_TYPE eLineType );
      //-----------------------------------------------------------------------
      /*!
      \if english
        The standard constructor is not publicly available.
      \endif
    
      \if german
        Der Standartkonstruktor steht nicht öffendlich zur Verfügung.
      \endif
      */
      Section();
      //-----------------------------------------------------------------------

    private:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The copy constructor is not available.
      \endif
    
      \if german
        Der Copy-Konstruktor steht nicht zur Verfügung.
      \endif
      */
      Section( const Section & );
      //-----------------------------------------------------------------------

    public:
      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the name of the INI section.
        \return The designation as string <B>without</B> "[" and "]".
      \endif
    
      \if german
        Liefert die Bezeichnung der INI-Sektion.
        \return Die Bezeichnung als String <B>ohne</B> "[" und "]".
      \endif
      */
      std::string getName() const;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Sets the text of the INI entry.
        \param[in] sName The name as string <B>without</B> "[" and "]".
      \endif
    
      \if german
        Setzt den Text des INI-Eintrags.
        \param[in] sName Die Bezeichnung als String <B>ohne</B> "[" und "]".
      \endif
      */
      void setName( const std::string & sName);
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns an Line from the #m_SektionList with respect to the pass parameters.
        \param[in,out] prRetVal passed reference into which the found object "comes".
        \param[in] nPosition Offset relative to...
        \param[in] eMode ...Position

        <B>Parameter examples:</B>
        \li Delivers first element from list: lPosition = 0; eMode = ptl::START
        \li Returns last element from list: lPosition = 0; eMode = ptl::END
        \li Returns current element from list: lPosition = 0; eMode = ptl::CURRENT_POS
        \li Returns next element from list: lPosition = 1; eMode = ptl::CURRENT_POS
        \li Returns previous element from list: lPosition = -1; eMode = ptl::CURRENT_POS

        \return A return < 0 indicates an error.
        \attention The object returned in prRetVal must <b>not</b> be removed
          from memory by the calling instance!
      \endif
    
      \if german
        Liefert einen Line aus der #m_SektionList in Bezug auf die ÜbergabeParameter.
        \param[in,out] prRetVal übergebene Referenz in welche das gefundene Objekt "hineinkommt"
        \param[in] nPosition Offset rellativ zur...
        \param[in] eMode ...Position

        <B>Parameterbeispiele:</B>
        \li Liefert erstes Element aus Liste: lPosition =  0; eMode = ptl::START
        \li Liefert letztes Element aus Liste: lPosition =  0; eMode = ptl::END
        \li Liefert aktuelles Element aus Liste: lPosition =  0; eMode = ptl::CURRENT_POS
        \li Liefert nächstes Element aus Liste: lPosition =  1; eMode = ptl::CURRENT_POS
        \li Liefert vorgeriges Element aus Liste: lPosition = -1; eMode = ptl::CURRENT_POS

        \return Eine Rückgabe < 0 zeigt einen Fehler an.
        \attention Das gelieferte Objekt in prRetVal darf <b>nicht</b> von der aufrufenden
          Instanz aus dem Speicher entfernt werden!
      \endif
      */
      int getSectionLine( Line *& prRetVal, std::size_t nPosition, ptl::SeekMode eMode );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Inserts the transferred entry into the #m_SektionList.
        \param[in] pIniLine The object of the class Line to be inserted, 
        \param[in] eInsertPosition Shows where the data object is inserted in the
          list is to be inserted. Possible parameters are ptl::InsertHEAD (on start),
          ptl::InsertLAST (at the end) or ptl::InsertCURRENT (at the current element).
        \param[in] eBeforeBehind Specifies whether the new element is before or
          after the, element specified in the eInsertPosition parameter. Possible
          Parameters are ptl::BEHIND and ptl::BEFORE
        \return A return value < 0 indicates an error.
        \attention The responsibility for the passed Line is taken by the object of
          this class, the calling instance must not remove the object from the
          memory.
      \endif
    
      \if german
        Fügt den übergebenen Eintrag in die #m_SektionList ein.
        \param[in] pIniLine Das einzufügende Objekt der Klasse Line, 
        \param[in] eInsertPosition Zeigt an, wohin das Datenobjekt in die Liste
          eingefügt werden soll. Mögliche Parameter sind ptl::InsertHEAD (am
          Anfang), ptl::InsertLAST (am Ende) oder ptl::InsertCURRENT (am
          aktuellen Element).
        \param[in] eBeforeBehind Gibt an, ob das neue Element vor oder hinter das,
          im Parameter eInsertPosition angegebene Element, gesetzt wird. Mögliche
          Parameter sind ptl::BEHIND und ptl::BEFORE
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
        \attention Die Verantwortung für die übergebene Line übernimmt das Objekt
          dieser Klasse, die aufrufende Instanz darf das Objekt nicht aus dem
          Speicher entfernen.
      \endif
      */
      int insertSectionLine(  Line * pIniLine, 
                              ptl::InsertPosition eInsertPosition = ptl::InsertCURRENT, 
                              ptl::VH eBeforeBehind = ptl::BEHIND );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Removes the entry from the #m_SektionList.

        If a subsequent element exists, it becomes the current element, otherwise
        the previous element, if it exists. The destructor is called from the
        remote entry.
        \param[in] eFirstLastCurrent The following values are possible: ptl::FIRST_POS,
          ptl::LAST_POS, (default) ptl::CURRENT_POS, ptl::NEXT_POS and ptl::PREVIOUS_POS.
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Entfernt den Eintrag aus der #m_SektionList.

        Wenn ein nachfolgendes Element vorhanden ist, wird dieses zum aktuellen Element,
        ansonsten das vorherige Element, wenn dieses vorhanden ist. Vom entfernten
        Eintrag wird der Destruktor aufgerufen.
        \param[in] eFirstLastCurrent Folgende Werte sind möglich: ptl::FIRST_POS, ptl::LAST_POS,
          (default) ptl::CURRENT_POS, ptl::NEXT_POS und ptl::PREVIOUS_POS.
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
      \endif
      */
      int removeSectionLine( ptl::ElementPosition eFirstLastCurrent = ptl::CURRENT_POS) ;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Removes all entries from the #m_SektionList.
        The destructor is called from each entry.
      \endif
    
      \if german
        Entfernt alle Einträge aus der #m_SektionList.
        Von jedem Eintrag wird der Destruktor aufgerufen.
      \endif
      */
      void removeAllSectionLines();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the section size from the current entries and itself (including
        the EOLines).
        \return The length of the section size.
      \endif
    
      \if german
        Liefert die Sektionsgrösse aus den aktuellen Einträgen und seiner selbst
        (inkl. den EOLines).
        \return Die Länge der Sektionsgrösse.
      \endif
      */
      std::size_t getSectionSize();
      //-----------------------------------------------------------------------
      /*!
      \if english
        This method returns a pointer to a Line from this section.
        \param[in,out] prRetVal The found Line
        \param[in] sValueVariable The searched variable or value
          \code{.unparsed}
          [386enh]
          woafont=app850.fon <- Variable (woafont) with value
          Test123 <- Value
          \endcode

        \return A return < 0 indicates an error.
        \attention The returned object in prRetVal must <b>not</b> be removed from memory
          by the calling instance!
      \endif
    
      \if german
        Diese Methode liefert einen Zeiger auf eine Line aus dieser Sektion.
        \param[in,out] prRetVal Die gefundene Line
        \param[in] sValueVariable Die gesuchte Variable bzw. Wert
          \code{.unparsed}
          [386enh]
          woafont=app850.fon  <- Variable (woafont) mit Wert
          Test123             <- Wert
          \endcode

        \return Eine Rückgabe < 0 zeigt einen Fehler an.
        \attention Das gelieferte Objekt in prRetVal darf <b>nicht</b> von der aufrufenden
          Instanz aus dem Speicher entfernt werden!
      \endif
      */
      int getKeyLine( Line *& prRetVal, const std::string & sValueVariable );
      //-----------------------------------------------------------------------
      /*!
      \if english
        This method returns a string corresponding to the value of the variable
        from the second parameter, from this section.
        \param[in,out] rsValue The found KeyValue as string.
        \param[in] sValueVariable The searched variable or value
        \return A return < 0 indicates an error.
      \endif
    
      \if german
        Diese Methode liefert einen String, welcher dem Wert der Variable
        aus dem zweiten Parameter entspricht, aus dieser Sektion.
        \param[in,out] rsValue Der gefundene KeyValue als String.
        \param[in] sValueVariable Die gesuchte Variable bzw. Wert
        \return Eine Rückgabe < 0 zeigt einen Fehler an.
      \endif
      */
      int getKeyValue( std::string & rsValue, const std::string sValueVariable );
      //-----------------------------------------------------------------------

    protected:
      //============== Methoden
      //-----------------------------------------------------------------------
      /*!
      \if english
        Appends its own section text and its own entries to the blob passed.
        \param[in,out] rBlob This blob is expanded.
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Hängt seinen eigenen Sektionstext und die eigenen Einträge an übergebenen
        Blob an.
        \param[in,out] rBlob Dieses Blob wird erweitert.
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
      \endif
      */
      int insertSectionInBlob( eut::Blob & rBlob );
      //-----------------------------------------------------------------------

    private:
      //============== Operatoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The assignment operator is not available.
      \endif
    
      \if german
        Der Zuweisungsoperator steht nicht zur Verfügung.
      \endif
      */
      Section & operator= ( const Section & );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von Clients von Klasse 'Typ2' verwendet wird */
    #pragma warning( default : 4251 )
  #endif
  //---------------------------------------------------------------------------
} // namespace ini
//-----------------------------------------------------------------------------
#endif
