#ifndef INC_EUEXCEPTION_H
#define INC_EUEXCEPTION_H
//-----------------------------------------------------------------------------
/*!
\file EuException.h
\brief
\if english
  Header for ErrorNException
\endif

\if german
  Header für ErrorNException
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTools module.

EuTools is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "EuToolTypes.h"
#include <stdexcept>
//-----------------------------------------------------------------------------
namespace eut
{
  //---------------------------------------------------------------------------
  /*! \brief
  \if english
    This exception class has an integer attribute for an error number in addition
    to the message.
  \endif

  \if german
    Diese Exception-Klasse hat zusätzlich zur Message ein Integer-Attribute
    für eine Fehlernummer.
  \endif

  \copyright © 2010 Helmut Jakoby
  \author Helmut Jakoby
  */
  class ErrorNException : public std::runtime_error
  {
    private:
      //============== Attribute
      //-------------------------------------------------------------------------
      /*!
      \if english
        The error number is in this attribute.
      \endif
    
      \if german
        Die FehlerNummer kommt in dieses Attribut.
      \endif
      */
      int m_iError;
        //---------------------------------------------------------------------
        /*!
        \if english
          The name of the file where the exception was trown.
        \endif

        \if german
          Der Name der Datei, in der die Exception ausgelöst wurde.
        \endif
        */
        char const * m_FileName;
        //---------------------------------------------------------------------
        /*!
        \if english
          The line number where the exception was trown.
        \endif

        \if german
          Die Zeilennummer, in der die Exception ausgelöst wurde.
        \endif
        */
        int m_iLineNumber;
      //-------------------------------------------------------------------------

    public:
      //============== De- & Konstruktoren
      //-------------------------------------------------------------------------
      /*!
      \if english
        Constructor with parameter passing.
        \param[in] _Message Error text.
        \param[in] iError Error code.
        \param[in] chFileName The name of the file where the exception was trown.
        \param[in] iLine The line number where the exception was trown.
      \endif
    
      \if german
        Konstruktor mit Parameterübergabe.
        \param[in] _Message Fehlertext.
        \param[in] iError Fehlercode.
        \param[in] chFileName Der Name der Datei, in der die Exception ausgelöst wurde.
        \param[in] iLine Die Zeilennummer, in der die Exception ausgelöst wurde.
      \endif
      */
      inline ErrorNException( const std::string & _Message, int iError, char const * chFileName, int iLine );
      //-------------------------------------------------------------------------
      /*!
      \if english
        Constructor with parameter passing.
        \param[in] _Message Error text.
        \param[in] iError Error code.
        \param[in] chFileName The name of the file where the exception was trown.
        \param[in] iLine The line number where the exception was trown.
      \endif
    
      \if german
        Konstruktor mit Parameterübergabe.
        \param[in] _Message Fehlertext.
        \param[in] iError Fehlercode.
        \param[in] chFileName Der Name der Datei, in der die Exception ausgelöst wurde.
        \param[in] iLine Die Zeilennummer, in der die Exception ausgelöst wurde.
      \endif
      */
      inline ErrorNException( const char *_Message, int iError, char const * chFileName, int iLine );
      //-------------------------------------------------------------------------
      /*!
      \if english
        Copy constructor.
        \param[in] rT The data is copied from this object.
      \endif
    
      \if german
        Copy-Konstruktor.
        \param[in] rT Von diesem Objekt werden die Daten übernommen.
      \endif
      */
      inline ErrorNException( const ErrorNException & rT );
      //-------------------------------------------------------------------------

      //============== Methoden
      //-------------------------------------------------------------------------
      /*!
      \if english
        Returns the error code.
        \return see above.
      \endif
    
      \if german
        Liefert den Fehlercode.
        \return s.o.
      \endif
      */
      inline int error() const;
      //-------------------------------------------------------------------------
      /*!
      \if english
        Returns the name of the file where the exception was trown.
        \return see above.
      \endif
    
      \if german
        Liefert den Name der Datei, in der die Exception ausgelöst wurde.
        \return s.o.
      \endif
      */
      inline char const * fileName() const;
      //-------------------------------------------------------------------------
      /*!
      \if english
        Returns the line number where the exception was trown.
        \return see above.
      \endif
    
      \if german
        Liefert die Zeilennummer, in der die Exception ausgelöst wurde.
        \return s.o.
      \endif
      */
      inline int lineNumber() const;
      //-------------------------------------------------------------------------

      //-------------------------------------------------------------------------
      /*!
      \if english
        Allocation operator which expects an ErrorNException.
        \param[in] rT The data is transferred from this object.
      \endif
    
      \if german
        Zuweisungsoperator welcher ein ErrorNException erwartet.
        \param[in] rT Von diesem Objekt werden die Daten übernommen.
      \endif
      */
      inline ErrorNException & operator= ( const ErrorNException & rT );
      //-------------------------------------------------------------------------
  };

  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    /* warning C4189: "t_iDEBUG_XXX": Lokale Variable ist initialisiert aber nicht referenziert */
    #pragma warning( disable : 4189 )
  #endif
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    /* Warnung: unused variable 't_iDEBUG_XXX' [-Wunused-variable] */
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wunused-variable"
  #endif
  //---------------------------------------------------------------------------
  inline  ErrorNException::ErrorNException( const std::string& _Message, int iError, char const * chFileName, int iLine )
  : std::runtime_error( _Message ),
    m_iError( iError ),
    m_FileName( chFileName ),
    m_iLineNumber( iLine )
  {
    #ifdef _DEBUG
      int  t_iDEBUG_BreakPoint = 1;
    #endif
  }
  //---------------------------------------------------------------------------
  inline  ErrorNException::ErrorNException( const char *_Message, int iError, char const * chFileName, int iLine )
  : std::runtime_error( _Message ),
    m_iError( iError ),
    m_FileName( chFileName ),
    m_iLineNumber( iLine )
  {
    #ifdef _DEBUG
      int  t_iDEBUG_BreakPoint = 1;
    #endif
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    /* warning C4189: "t_iDEBUG_XXX": Lokale Variable ist initialisiert aber nicht referenziert */
    #pragma warning( default : 4189 )
  #endif
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    /* Warnung: unused variable 't_iDEBUG_XXX' [-Wunused-variable] */
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  inline  ErrorNException::ErrorNException( const ErrorNException & rT )
  : std::runtime_error( rT ),
    m_iError( rT.m_iError ),
    m_FileName( rT.m_FileName ),
    m_iLineNumber( rT.m_iLineNumber )
  {
  }
  //---------------------------------------------------------------------------
  inline int ErrorNException::error() const
  {
    return m_iError;
  }
  //---------------------------------------------------------------------------
  inline char const * ErrorNException::fileName() const
  {
    return m_FileName;
  }
  //---------------------------------------------------------------------------
  inline int ErrorNException::lineNumber() const
  {
    return m_iLineNumber;
  }
  //---------------------------------------------------------------------------
  inline ErrorNException & ErrorNException::operator= ( const ErrorNException & rT )
  {
    if( this != & rT )
    {
      m_iError = rT.m_iError;
      m_FileName = rT.m_FileName;
      m_iLineNumber = rT.m_iLineNumber;
      std::runtime_error::operator=( rT );
    }
    return *this;
  }
  //---------------------------------------------------------------------------
} // namespace eut
#endif
