//-----------------------------------------------------------------------------
/*!
\file EuBlob.cpp
\brief Source for Blob
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTools module.

EuTools is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "EuBlob.h"
#include "EuException.h"
#include <cstring>
//-----------------------------------------------------------------------------
#ifdef _DEBUG_MEMORY_SPY
  #include <MilDebugNew.h>
#endif
//-----------------------------------------------------------------------------
namespace eut
{
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // 't_BadbAllocEx': Unreferenzierte lokale Variable
    #pragma warning( disable : 4101 )
  #endif
  //---------------------------------------------------------------------------
  Blob::Blob()
  : m_pBlob(NULL_PTR),
    m_nSize(0),
    m_nCurrentPosition(0),
    m_bMemResponsible(true)
  {
  }
  //---------------------------------------------------------------------------
  Blob::Blob( char * pData, std::size_t nDataSize, bool bCopy )
  : m_pBlob(NULL_PTR),
    m_nSize(0),
    m_nCurrentPosition(0),
    m_bMemResponsible(true)
  {
    int  t_iErr = 0;
  
    t_iErr = this->setData( pData, nDataSize, bCopy );
    if (t_iErr)
    {
      throw eut::ErrorNException( "Blob ctor", ERR_NEW, __FILE__, __LINE__ );
    }
  }
  //---------------------------------------------------------------------------
  Blob::Blob( const Blob & rBlob )
  : m_pBlob(NULL_PTR),
    m_nSize(0),
    m_nCurrentPosition(0),
    m_bMemResponsible(true)
  {
    int  t_iErr = 0;
  
    t_iErr = this->setData( rBlob.m_pBlob, rBlob.m_nSize, m_bMemResponsible );
    if (t_iErr)
    {
      throw eut::ErrorNException( "Blob copy-ctor", ERR_NEW, __FILE__, __LINE__ );
    }
  }
  //---------------------------------------------------------------------------
  Blob::~Blob()
  {
    if ( m_pBlob && m_bMemResponsible )
    {
      delete[] m_pBlob;
    }
  }
  //---------------------------------------------------------------------------
  int Blob::swap( Blob & rSwapBlob )
  {
    int   t_iErr = 0;
  
    if ( m_bMemResponsible == rSwapBlob.m_bMemResponsible )
    {
      char         * t_pTempThisBuffer = m_pBlob;
      std::size_t    t_nTempThisSize = m_nSize,
                     t_nTempCurrentPosition = m_nCurrentPosition;
    
      m_pBlob =  rSwapBlob.m_pBlob;
      m_nSize =  rSwapBlob.m_nSize;
      m_nCurrentPosition =  rSwapBlob.m_nCurrentPosition;
    
      rSwapBlob.m_pBlob =  t_pTempThisBuffer;
      rSwapBlob.m_nSize =  t_nTempThisSize;
      rSwapBlob.m_nCurrentPosition =  t_nTempCurrentPosition;
    }
    else
    {
      t_iErr = ERR_WRONG_TYPE;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  int Blob::setData( char * pData, std::size_t nDataSize, bool bCopy )
  {
    int  t_iErr = 0;
  
    m_nCurrentPosition = 0;
    if ( m_pBlob )
    {
      if ( m_bMemResponsible )  delete[] m_pBlob;
      m_pBlob = NULL_PTR;
      m_nSize = 0;
    }
    if ( pData && nDataSize )
    {
      if ( bCopy )
      {
        try
        {
          m_pBlob = new char [nDataSize];
        }
        catch ( const std::bad_alloc & )
        {
          m_pBlob = NULL_PTR;
          t_iErr = ERR_NEW;
        }
        if ( m_pBlob )
        {
          std::memcpy( m_pBlob, pData, nDataSize );
          m_nSize = nDataSize;
        }
      }
      else
      {
        m_pBlob = pData;
        m_nSize = nDataSize;
      }
      m_bMemResponsible = bCopy;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  void Blob::setMemResponsible( bool bActiv )
  {
    m_bMemResponsible = bActiv;
  }
  //---------------------------------------------------------------------------
  bool Blob::isMemResponsible()
  {
    return m_bMemResponsible;
  }
  //---------------------------------------------------------------------------
  void Blob::clear()
  {
    if ( m_pBlob )
    {
      if ( m_bMemResponsible ) delete[] m_pBlob;
      m_pBlob = NULL_PTR;
    }
    m_nSize = 0;
    m_nCurrentPosition = 0;
  }
  //---------------------------------------------------------------------------
  void Blob::setNULL()
  {
    m_pBlob = NULL_PTR;
    m_nSize = 0;
    m_nCurrentPosition = 0;
  }
  //---------------------------------------------------------------------------
  size_t Blob::getData( char *& prBlob, bool bCopy )
  {
    if ( m_pBlob && m_nSize )
    {
      if ( bCopy )
      {
        try
        {
          prBlob = new char [m_nSize];
        }
        catch ( const std::bad_alloc & )
        {
          throw eut::ErrorNException( "Blob::getData(char *& prBlob, bool bCopy)", ERR_NEW, __FILE__, __LINE__ );
        }
        std::memcpy( prBlob, m_pBlob,  m_nSize );
      }
      else
      {
        prBlob = m_pBlob;
      }
    }
    return m_nSize;
  }
  //---------------------------------------------------------------------------
  int Blob::getData(  char *& prBlob, std::size_t nStartPosition, std::size_t nSize)
  {
    int  t_iErr = 0;
  
    prBlob = NULL_PTR;
    if ( nSize && ( nStartPosition > 0 ) && ( nStartPosition < m_nSize ) )
    {
      --nStartPosition;
      if ( nSize > m_nSize - nStartPosition )
      {
        nSize = m_nSize - nStartPosition;
      }
      // neues Blob erstellen
      try
      {
        prBlob = new char [nSize];
      }
      catch ( const std::bad_alloc & )
      {
        prBlob = NULL_PTR;
        t_iErr = ERR_NEW;
      }
      if ( prBlob )
      {
        std::memcpy( prBlob, m_pBlob + nStartPosition, nSize );
      }
    }
    else
    {
      t_iErr = ERR_RANGE;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  int Blob::getBlob(  Blob *& prBlob,
                      std::size_t nStartPosition,
                      std::size_t nSize )
  {
    int      t_iErr = 0;
    char  *  pNewBlob = NULL_PTR;
  
    prBlob = NULL_PTR;
    t_iErr = getData( pNewBlob, nStartPosition, nSize );
    if ( ! t_iErr )
    {
      prBlob = new Blob( pNewBlob, nSize, false );
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  char Blob::getChar( std::size_t nPosition )
  {
    if ( nPosition >= 1 )
    {
      --nPosition;
      if ( nPosition < m_nSize )
      {
        return m_pBlob[nPosition];
      }
    }
    return 0;
  }
  //---------------------------------------------------------------------------
  std::size_t Blob::getSize() const
  {
    return m_nSize;
  }
  //-------------------------------------------------------------------
  bool Blob::findForward( std::size_t & nFoundPosition,
                          const char * pSearchMask,
                          std::size_t nSearchMaskSize,
                          bool bSearchAtTheBeginning)
  {
    bool    t_bFound = false;
  
    if ( bSearchAtTheBeginning )
    {
      // An den start
      m_nCurrentPosition = 0;
    }
    else
    {
      // einen weiter
      ++m_nCurrentPosition;
    }
    if ( m_pBlob
         &&
         pSearchMask
         &&
         m_nSize
         &&
         nSearchMaskSize
         &&
         ! ( m_nCurrentPosition > m_nSize ) )
    {
      while  (  !t_bFound
              &&
              ( ( m_nCurrentPosition + nSearchMaskSize ) <= m_nSize ) )
      {
        std::size_t   t_uiMaskPosition = 0;
        
        while  (  ( (m_nCurrentPosition + nSearchMaskSize ) <= m_nSize )
                &&
                ( m_pBlob[m_nCurrentPosition] != pSearchMask[t_uiMaskPosition] ) )
        {
          ++m_nCurrentPosition;
        }
        // wenn noch nicht am Ende
        if ( ( m_nCurrentPosition + nSearchMaskSize ) <= m_nSize )
        {
          // wenn nur 1 Zeichen geprüft werden soll; ende mit suchen
          if ( nSearchMaskSize == 1 )
          {
            t_bFound = true;
            nFoundPosition = m_nCurrentPosition + 1;
          }
          else
          {
            bool    t_bKeepLooking = false;
            
            // Rest vergleichen
            do
            {
              t_bKeepLooking = false;
              if ( m_pBlob[++m_nCurrentPosition] == pSearchMask[++t_uiMaskPosition] )
              {
                t_bKeepLooking = true;
              }
            }
            while ( t_bKeepLooking && ( t_uiMaskPosition < ( nSearchMaskSize - 1 ) ) );
            if ( t_bKeepLooking )
            {
              t_bFound = true;
              nFoundPosition = m_nCurrentPosition - t_uiMaskPosition + 1;
            }
          }
        }
      }
    }
    return t_bFound;
  }
  //---------------------------------------------------------------------------
  bool Blob::findForward( std::size_t & nFoundPosition,
                          const char * pSearchMask,
                          std::size_t nSearchMaskSize )
  {
    return findForward( nFoundPosition, pSearchMask, nSearchMaskSize, true );
  }
  //---------------------------------------------------------------------------
  bool Blob::findForwardNext( std::size_t & nFoundPosition,
                              const char * pSearchMask,
                              std::size_t nSearchMaskSize )
  {
    return findForward( nFoundPosition, pSearchMask, nSearchMaskSize, false );
  }
  //---------------------------------------------------------------------------
  bool Blob::findBackward( std::size_t & nFoundPosition,
                           const char * pSearchMask,
                           std::size_t nSearchMaskSize,
                           bool bSearchAtTheEnd )
  {
    bool          t_bAtTheStart = false,
                  t_bFound = false;
  
    if ( bSearchAtTheEnd )
    {
      // Ans Ende minus Länge der zu suchenden Maske
      m_nCurrentPosition = m_nSize - nSearchMaskSize;
    }
    else
    {
      if ( m_nCurrentPosition == 0 )
      {
        t_bAtTheStart = true;
      }
      else
      {
        // einen weiter nach  vorne
        --m_nCurrentPosition;
      }
    }
    if ( ! t_bAtTheStart
         &&
         m_pBlob
         &&
         pSearchMask
         &&
         m_nSize
         &&
         nSearchMaskSize
         &&
         ! ( m_nCurrentPosition > m_nSize ) )
    {
      while (  ! t_bFound
              &&
              ! t_bAtTheStart
              &&
              ( ( m_nCurrentPosition + nSearchMaskSize ) <= m_nSize ) )
      {
        std::size_t   t_uiMaskPosition = 0;
        while  (  ! t_bAtTheStart
                &&
                ( ( m_nCurrentPosition + nSearchMaskSize ) <= m_nSize )
                &&
                ( m_pBlob[m_nCurrentPosition] != pSearchMask[t_uiMaskPosition] ) )
        {
          if (m_nCurrentPosition == 0)
          {
            t_bAtTheStart = true;
          }
          --m_nCurrentPosition;
        }
        // wenn noch nicht am Anfang
        if ( ! t_bAtTheStart )
        {
          // wenn nur 1 Zeichen geprüft werden soll; ende mit suchen
          if ( nSearchMaskSize == 1 )
          {
            t_bFound = true;
            nFoundPosition = m_nCurrentPosition + 1;
          }
          else
          {
            std::size_t   t_nTempPosition = m_nCurrentPosition;
            bool          t_bKeepLooking = false;
            // Rest vergleichen
            do
            {
              t_bKeepLooking = false;
              if ( m_pBlob[++t_nTempPosition] == pSearchMask[++t_uiMaskPosition] )
              {
                t_bKeepLooking = true;
              }
            }
            while ( t_bKeepLooking && ( t_uiMaskPosition < ( nSearchMaskSize - 1 ) ) );
            if ( t_bKeepLooking )
            {
              t_bFound = true;
              nFoundPosition = t_nTempPosition - t_uiMaskPosition + 1;
            }
            else
            {
              if ( m_nCurrentPosition == 0 )
              {
                t_bAtTheStart = true;
              }
              --m_nCurrentPosition;
            }
          }
        }
      }
    }
    return t_bFound;
  }
  //---------------------------------------------------------------------------
  bool Blob::findBackward( std::size_t & nFoundPosition,
                           const char * pSearchMask,
                           std::size_t nSearchMaskSize )
  {
    return findBackward( nFoundPosition, pSearchMask, nSearchMaskSize, true );
  }
  //---------------------------------------------------------------------------
  bool Blob::findBackwardNext( std::size_t & nFoundPosition,
                               const char * pSearchMask,
                               std::size_t nSearchMaskSize)
  {
    return findBackward( nFoundPosition, pSearchMask, nSearchMaskSize, false );
  }
  //---------------------------------------------------------------------------
  int Blob::setCurrentPosition( std::size_t nNewPosition )
  {
      int t_iErr = 0;
  
      if ( nNewPosition < 1 || nNewPosition > m_nSize )
      {
        t_iErr = ERR_RANGE;
      }
      else
      {
        m_nCurrentPosition = nNewPosition - 1;
      }
      return t_iErr;
  }
  //---------------------------------------------------------------------------
  std::size_t Blob::getCurrentPosition() const
  {
    return m_nCurrentPosition + 1;
  }
  //---------------------------------------------------------------------------
  int Blob::append( const char * pBuffer, std::size_t nBufferSize )
  {
    return this->insert( pBuffer, nBufferSize, this->getSize() + 1 );
  }
  //---------------------------------------------------------------------------
  int Blob::append( const Blob * pBlob )
  {
    return this->insert( pBlob, this->getSize() + 1 );
  }
  //---------------------------------------------------------------------------
  int Blob::append( const Blob & rBlob )
  {
    return this->insert( rBlob, this->getSize() + 1 );
  }
  //---------------------------------------------------------------------------
  int Blob::insert( const char * pBuffer,
                    std::size_t nBufferSize,
                    std::size_t nBufferPosition)
  {
    int      t_iErr = 0;
    char  *  t_pNewBuffer = NULL_PTR,
          *  t_pAttachBuffer = NULL_PTR;
  
    if ( ! m_bMemResponsible ) return ERR_METHOD_VORBIDDEN;
    // Wenn uiNumberRemoveBytes = 0, dann nichts tun
    if ( ! nBufferSize ) return 0;
    --nBufferPosition;
    if ( pBuffer && nBufferSize && ( nBufferPosition <= m_nSize ) )
    {
      // neues Blob erstellen
      try
      {
        t_pNewBuffer = new char [m_nSize + nBufferSize];
      }
      catch ( const std::bad_alloc & )
      {
        t_pNewBuffer = NULL_PTR;
        t_iErr = ERR_NEW;
      }
      if ( ! t_iErr )
      {
        // wenn uiBufferPointer > 0, dann vorderen Teil in neues Blob
        if (nBufferPosition > 0)
        {
          std::memcpy(t_pNewBuffer, m_pBlob, nBufferPosition);
        }
        // wenn uiBufferPointer nicht am Ende, Hinteren Teil sichern
        if ( nBufferPosition != m_nSize )
        {
          try
          {
            t_pAttachBuffer = new char [m_nSize - nBufferPosition];
          }
          catch ( const std::bad_alloc & )
          {
            t_pAttachBuffer = NULL_PTR;
            t_iErr = ERR_NEW;
          }
          if ( t_pAttachBuffer )
          {
            std::memcpy( t_pAttachBuffer, m_pBlob + nBufferPosition, m_nSize - nBufferPosition );
          }
        }
        if ( ! t_iErr )
        {
          // pBuffer an neues Blob anhängen
          std::memcpy( t_pNewBuffer + nBufferPosition, pBuffer, nBufferSize );
          // wenn was hinten drannzuhängen
          if (t_pAttachBuffer)
          {
            std::memcpy(  t_pNewBuffer + nBufferPosition + nBufferSize,
                          t_pAttachBuffer,
                          m_nSize - nBufferPosition );
            delete[] t_pAttachBuffer;
          }
          // setzen ohne kopieren; kann keinen Fehler geben
          this->setData( t_pNewBuffer, m_nSize + nBufferSize, false );
          // aber m_bMemResponsible wieder setzen
          m_bMemResponsible = true;
        }
        else
        {
          delete[] t_pNewBuffer;
        }
      }
    }
    else
    {
      t_iErr = ERR_RANGE;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  int Blob::insert( const Blob * pBlob, std::size_t nBufferPosition )
  {
    int   t_iErr = 0;
  
    if ( ! m_bMemResponsible ) return ERR_METHOD_VORBIDDEN;
    if ( pBlob )
    {
      Blob  * t_pConstCastBlob = const_cast<Blob*>( pBlob );
      char  * t_pBufferPointer = NULL_PTR;

      std::size_t   t_nBufferSize = t_pConstCastBlob->getData( t_pBufferPointer, false );
      t_iErr = this->insert( t_pBufferPointer, t_nBufferSize, nBufferPosition );
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  int Blob::insert( const Blob & rBlob, std::size_t nBufferPosition )
  {
    return this->insert( & rBlob, nBufferPosition );
  }
  //---------------------------------------------------------------------------
  int Blob::remove( std::size_t nNumberRemoveBytes,
                    std::size_t nBufferPosition )
  {
    int      t_iErr = 0;
    char  *  pNewBlob = NULL_PTR;
    
    if ( ! m_bMemResponsible ) return ERR_METHOD_VORBIDDEN;
    // Wenn uiNumberRemoveBytes = 0, dann nichts tun
    if ( ! nNumberRemoveBytes ) return 0;
    --nBufferPosition;
    if ( nNumberRemoveBytes && ( nBufferPosition <= m_nSize ) )
    {
      // falls Ende ausserhalb Range; anpassen
      if ( nNumberRemoveBytes > ( m_nSize - nBufferPosition ) )
      {
        nNumberRemoveBytes = m_nSize - nBufferPosition;
      }
      // neues Blob erstellen
      try
      {
        pNewBlob = new char [m_nSize - nNumberRemoveBytes];
      }
      catch ( const std::bad_alloc & )
      {
        pNewBlob = NULL_PTR;
        t_iErr = ERR_NEW;
      }
      if ( ! t_iErr )
      {
        // wenn uiBufferPointer > 0, dann vorderen Teil in neues Blob
        if ( nBufferPosition > 0 )
        {
          std::memcpy( pNewBlob, m_pBlob, nBufferPosition );
        }
        // jetzt ab Zeiger + Anzahl anhängen wenn noch was da
        if ( ( m_nSize - nBufferPosition - nNumberRemoveBytes ) > 0 )
        {
          std::memcpy(  pNewBlob + nBufferPosition,
                        m_pBlob + nNumberRemoveBytes + nBufferPosition,
                        ( m_nSize - nBufferPosition - nNumberRemoveBytes ) );
        }
        // setzen ohne kopieren; kann keinen Fehler geben
        this->setData( pNewBlob, m_nSize - nNumberRemoveBytes, false );
        // und m_bMemResponsible zurücksetzen
        m_bMemResponsible = true;
      }
    }
    else
    {
      t_iErr = ERR_RANGE;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  int Blob::replace(  const char * pOldContent,
                      std::size_t pOldContentSize,
                      const char * pNewContent,
                      std::size_t pNewContentSize )
  {
    int           t_iErr = 0,
                  t_iRetval = 0;
    std::size_t   t_uiFoundPos = 0;
    bool          t_bFound = this->findForward( t_uiFoundPos, pOldContent, pOldContentSize );

    if ( t_bFound )
    {
      t_iErr = this->remove( pOldContentSize, t_uiFoundPos );
      if ( ! t_iErr )
      {
        t_iErr = this->insert( pNewContent, pNewContentSize, t_uiFoundPos );
        if ( ! t_iErr )
        {
          this->setCurrentPosition( t_uiFoundPos + pNewContentSize - 1 );
          t_iRetval = 1;
        }
        else
        {
          t_iErr = ERR_INSERT;
        }
      }
    }
    return ( ! t_iErr ? t_iRetval : t_iErr );
  }
  //---------------------------------------------------------------------------
  int Blob::replaceAll( const char * pOldContent,
                        std::size_t pOldContentSize,
                        const char * pNewContent,
                        std::size_t pNewContentSize )
  {
    int           t_iErr = 0,
                  t_iRetval = 0;
    std::size_t   t_uiFoundPos = 0;
    bool          t_bFound = this->findForward( t_uiFoundPos, pOldContent, pOldContentSize );

    while ( t_bFound )
    {
      t_iErr = this->remove( pOldContentSize, t_uiFoundPos );
      if ( ! t_iErr )
      {
        t_iErr = this->insert( pNewContent, pNewContentSize, t_uiFoundPos );
        if ( ! t_iErr )
        {
          this->setCurrentPosition( t_uiFoundPos + pNewContentSize - 1 );
          ++t_iRetval;
        }
        else
        {
          t_iErr = ERR_INSERT;
          break;
        }
      }
      else
      {
        t_iErr = 0; //NOTHING_TO_REMOVE
        break;
      }
      t_bFound = this->findForwardNext( t_uiFoundPos, pOldContent, pOldContentSize );
    } // while ( t_bFound )
    return ( ! t_iErr ? t_iRetval : t_iErr );
  }
  //---------------------------------------------------------------------------
  Blob & Blob::operator+= ( const Blob & rBlob )
  {
    int  t_iErr = this->append( rBlob );
  
    if ( t_iErr )
    {
      throw eut::ErrorNException( "Blob operator+=", t_iErr, __FILE__, __LINE__ );
    }
    return *this;
  }
  //---------------------------------------------------------------------------
  Blob & Blob::operator= ( const Blob & rBlob )
  {
    if ( this != & rBlob )
    {
      int   t_iErr = this->setData( rBlob.m_pBlob, rBlob.m_nSize, rBlob.m_bMemResponsible );
      m_nCurrentPosition = rBlob.m_nCurrentPosition;
      if ( t_iErr )
      {
        throw eut::ErrorNException( "Blob operator=", ERR_NEW, __FILE__, __LINE__ );
      }
    }
    return *this;
  }
  //---------------------------------------------------------------------------
  Blob & Blob::operator= ( int )
  {
    this->clear();
    return *this;
  }
  //---------------------------------------------------------------------------
  int Blob::operator== ( const Blob & rBlob ) const
  {
    // zuerst Größe prüfen (geht schneller wenn ungleich)
    if ( m_nSize == rBlob.m_nSize )
    {
      // wenn diese gleich, dann Blob prüfen
      int   iResult = std::memcmp( m_pBlob, rBlob.m_pBlob, m_nSize );
      if ( iResult == 0 )
      {
        return 1;
      }
    }
    return 0;
  }
  //---------------------------------------------------------------------------
  int Blob::operator!= ( const Blob & rBlob ) const
  {
    if ( *this == rBlob )
    {
      return 0;
    }
    else
    {
      return 1;
    }
  }
  //---------------------------------------------------------------------------
  bool Blob::operator> ( const Blob & rBlob )
  {
    // Wenn m_nSize von übergebenem Blob kleiner...
    if ( m_nSize > rBlob.m_nSize )
    {
      // dann reicht das für größer
      return true;
    }
    return false;
  }
  //---------------------------------------------------------------------------
  bool Blob::operator< ( const Blob & rBlob )
  {
    if ( m_nSize < rBlob.m_nSize )
    {
      return false;
    }
    else
    {
      return true;
    }
  }
  //---------------------------------------------------------------------------
  bool Blob::operator>= ( const Blob & rBlob )
  {
    if ( m_nSize >= rBlob.m_nSize )
    {
      return true;
    }
    else
    {
      return false;
    }
  }
  //---------------------------------------------------------------------------
  bool Blob::operator<= ( const Blob & rBlob )
  {
    if ( m_nSize <= rBlob.m_nSize )
    {
      return true;
    }
    else
    {
      return false;
    }
  }
  //---------------------------------------------------------------------------
  Blob::operator int () const
  {
    if ( m_nSize )
    {
      return 1;
    }
    else
    {
      return 0;
    }
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // 't_BadbAllocEx': Unreferenzierte lokale Variable
    #pragma warning( default : 4101 )
  #endif
  //---------------------------------------------------------------------------
} // namespace eut
