#ifndef INC_PTLATDVLIST_H
#define INC_PTLATDVLIST_H
//-----------------------------------------------------------------------------
/*!
\file PtlATDVList.h
\brief
\if english
  Header for ATDVList
\endif

\if german
  Header für ATDVList
\endif
\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the PtlContainer module.

PtlContainer is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "PtlListTypes.h"
#include "PtlVDVList.h"
//-----------------------------------------------------------------------------
namespace ptl
{
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    Abstract superclass for template lists that include a pointer to objects or
    objects as copies in the list.
  \endif

  \if german
    Abstrakte Oberklasse für Template-Listen, welche eine Zeiger auf Objekte oder
    Objekte als Kopien in die Liste aufnehmen.
  \endif

  \copyright © 1995 Helmut Jakoby
  \author Helmut Jakoby
  */
  class ATDVList
  {
    protected:
      //============== Attribute ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        Pointer to VDVList, which works with void pointers.
      \endif
    
      \if german
        Zeiger auf VDVList, welche mit void-Pointern arbeitet.
      \endif
      */
      VDVList * m_VDVListe;
      //-----------------------------------------------------------------------

    public:
      //+ ============ Konstruktoren ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor with optional parameter passing.
        List is empty at first. With the parameter eMode you can instantiate a list,
        which includes data objects several times (no parameter or #DEFAULT)
        or only once (#UNIQUE) and/or should be the owner of the data objects
        (#RESPONSIBLE).
      \endif
    
      \if german
        Konstruktor mit optionaler Parameterübergabe.
        Liste ist erst einmal leer. Durch Parameter eMode kann eine Liste instanziiert
        werden, welche Datenobjekte mehrmals (keinen Parameter oder #DEFAULT) oder
        nur einmalig (#UNIQUE) aufnimmt und/oder Besitzer der Datenobjekte sein
        soll (#RESPONSIBLE).
      \endif
      */
      ATDVList( ListMode eMode = DEFAULT );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor, emptying list.
      \endif
    
      \if german
        Destruktor, Liste wird geleert.
      \endif
      */
      virtual ~ATDVList();
      //-----------------------------------------------------------------------

    private:
      //+ ============ Konstruktoren ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        Copy constructor is not available. This is to prevent a programmer from
        passing an object from this class to a function, for example, by value.
      \endif
    
      \if german
        Copy-Konstruktor steht nicht zur Verfügung. Damit soll verhindern werden,
        dass ein Programmierer ein Objekt aus dieser Klasse z.B. by value an eine
        Funktion übergibt.
      \endif
      */
      ATDVList( const ATDVList & );
      //-----------------------------------------------------------------------

    public:
      //+ ============ Methoden ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        This method allows you to cache the pointer to the current element. This
        allows e.g. to work through the list (which moves the current pointer) and
        then restore the current pointer.
        \param[in] eWhich Specifies whether the pointer to the current DV-element
          is buffered (#M) or restored (#RM).
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Diese Methode ermöglicht das Zwischenspeichern des Zeigers auf das aktuelle
        Element. Damit kann z.B. die Liste durchgearbeitet werden (welches den
        aktuellen Zeiger verschiebt) und danach der aktuelle Zeiger restauriert werden.
        \param[in] eWhich Gibt an, ob der Zeiger auf das aktuelle DVElement
          zwischengespeichert (#M) oder restauriert (#RM) wird.
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
      \endif
      */
      inline int currentElementCache( ElementCache eWhich );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the last error code.

        Should be called after insert methods to check if errors occurred.

        If errors #INSTANTIATE_DVELEMENT occurred, the instance should be
        deleted if no other error handling is implemented in subclasses.
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Liefert den letzten Fehler-Code.
      
        Sollte nach Einfügemethoden aufgerufen werden, um zu überprüfen, ob Fehler
        aufgetreten sind.

        Ist Fehler #INSTANTIATE_DVELEMENT aufgetreten, sollte die
        Instanz gelöscht werden, wenn keine andere Fehlerbehandlung in Unterklassen
        implementiert ist.
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
      \endif
      \sa ERRORS
      */
      inline int getErrorCode();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the pointer to the capped VDVList. Used by iterator classes,
        for example, to work in this VDVList.
        \return The referenced VDVList.
      \endif
    
      \if german
        Liefert den Zeiger auf die gekappselte VDVList. Wird z.B. von Iteratorklassen
        genutzt, um dann in dieser VDVList arbeiten zu können.
        \return Die referenzierte VDVList.
      \endif
      */
      inline VDVList * getVDVList();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the number of elements contained in the list.
        \return The number of elements

        <B>Example:</B>
      \endif
    
      \if german
        Liefert die Anzahl der in Liste enthaltenen Elemente.
        \return Die Anzahl der Elemente

        <B>Beispiel:</B>
      \endif
      \code{.cpp}
      ptl::AVDVList myList;
      std::size_t nSize = myListe.getSize()
      \endcode
      */
      inline std::size_t getSize();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the List mode.
        \return The ListMode ptl::RESPONSIBLE is only possible for template
        lists that use this class. A combination of the values can also be supplied.
        \sa ListMode

        <B>Example:</B>
      \endif
    
      \if german
        Liefert den ListModus.
        \return Der ListMode ptl::RESPONSIBLE ist nur bei Templatelisten
        möglich, welche diese Klasse nutzen. Es kann auch eine Kombination der
        Werte geliefert werden.
        \sa ListMode

        <B>Beispiel:</B>
      \endif
      \code{.cpp}
      ptl::AVDVList  myList;
      ...
      // if myList with UNIQUE-Bit...
      if (myList.getListMode() & ptl::UNIQUE)
      {
        // ..do something...
      }
      \endcode
      \if english
        <b>Here pure virtual, so this class is abstract.</b>
      \endif
    
      \if german
        <b>Hier pur virtual damit diese Klasse abstrakt ist.</b>
      \endif
      */
      virtual inline ListMode getListMode() = 0;
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
  inline std::size_t ATDVList::getSize()
  {
    return m_VDVListe->getSize();
  }
  //---------------------------------------------------------------------------
  inline VDVList * ATDVList::getVDVList()
  {
    return m_VDVListe;
  }
  //---------------------------------------------------------------------------
  inline int ATDVList::getErrorCode()
  {
    return m_VDVListe->getErrorCode();
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // Der Enumerationstyp "xxx" umfasst keine Bereichseinschränkung. Geben Sie "enum class" Vorrang vor "enum" (Enum.3)
    #pragma warning( disable : 26812 )
  #endif
  //---------------------------------------------------------------------------
  inline int ATDVList::currentElementCache( ElementCache eWhich )
  {
    return m_VDVListe->currentElementCache( eWhich );
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // Der Enumerationstyp "xxx" umfasst keine Bereichseinschränkung. Geben Sie "enum class" Vorrang vor "enum" (Enum.3)
    #pragma warning( default : 26812 )
  #endif
  //---------------------------------------------------------------------------
} // namespace ptl
#endif
