#ifndef INC_PTLLISTTYPES_H
#define INC_PTLLISTTYPES_H

//-----------------------------------------------------------------------------
/*!
\file PtlListTypes.h
\brief
\if english
  For each library, here 'PtlContainer' there is a type file.

  This file is usually included by every header from the library.
\endif

\if german
  Für jede Bibliothek, hier 'PtlContainer' gibt es eine Typen-Datei.

  Diese Datei wird in der Regel von jeder Header eingebunden.
\endif
\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the PtlContainer module.

PtlContainer is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "PtlListErrors.h"
#include "EuDef_NULL_PTR.h"
//-----------------------------------------------------------------------------
namespace ptl
{
  //---------------------------------------------------------------------------
  /*! 
  \enum ElementPosition

    \if english
      For DVLists, which element is meant by Get and Remove methods.
    \endif

    \if german
      Für DVListen, welches Element gemeint ist bei Get- und Remove-Methoden.
    \endif
  */
  /*!
  \var ElementPosition::FIRST_POS
    \if english
      The first element in the container is meant.
    \endif

    \if german
      Das erste Element im Container ist gemeint.
    \endif

  \var ElementPosition::LAST_POS
    \if english
      The last element in the container is meant.
    \endif
  
    \if german
      Das letzte Element im Container ist gemeint.
    \endif

  \var ElementPosition::CURRENT_POS
    \if english
      The current element in the container is meant.
    \endif
  
    \if german
      Das aktuelle Element im Container ist gemeint.
    \endif

  \var ElementPosition::NEXT_POS
    \if english
      The next element after the current element in the container is meant.
    \endif
  
    \if german
      Das nächste Element nach dem aktuellen Element im Container ist gemeint.
    \endif

  \var ElementPosition::PREVIOUS_POS
    \if english
      The element before the current element in the container is meant.
    \endif
  
    \if german
      Das Element vor dem aktuellen Element im Container ist gemeint.
    \endif
  */
  enum ElementPosition 
  {
    FIRST_POS,
    LAST_POS,
    CURRENT_POS,
    NEXT_POS,
    PREVIOUS_POS
  };
  //---------------------------------------------------------------------------
  /*! \enum VH
    \if english
      For DVLists, whether to insert before or after the #InsertPosition.
    \endif
  
    \if german
      Für DVListen, ob beim Einfügen vor oder hinter die #InsertPosition eingefügt werden soll.
    \endif
  */
  /*!
  \var VH::BEFORE
    \if english
      The element is inserted into the container before the #InsertPosition.
    \endif
  
    \if german
      Das Element wird vor der #InsertPosition in den Container eingefügt.
    \endif

  \var VH::BEHIND
    \if english
      The element is inserted into the container after the #InsertPosition.
    \endif
  
    \if german
      Das Element wird hinter der #InsertPosition in den Container eingefügt.
    \endif
  */
  enum VH
  {
    BEFORE,
    BEHIND
  };
  //---------------------------------------------------------------------------
  /*! \enum InsertPosition
    \if english
      For DVLists to specify an insertion point in conjunction with #VH.
    \endif
  
    \if german
      Für DVListen um eine Einfügeposition in Verbindung mit #VH zu bestimmen.
    \endif
    \sa enum VH
  */
  /*!
  \var InsertPosition::InsertHEAD
    \if english
      The insertion point is before or after the first element of the list.
    \endif
  
    \if german
      Die Einfügeposition ist vor bzw. nach dem ersten Element der Liste.
    \endif

  \var InsertPosition::InsertLAST
    \if english
      The insertion point is before or after the last element of the list.
    \endif
  
    \if german
      Die Einfügeposition ist vor bzw. nach dem letzten Elemet der Liste.
    \endif

  \var InsertPosition::InsertCURRENT
    \if english
      The insertion point is before or after the current element of the list.
    \endif
  
    \if german
      Die Einfügeposition ist vor bzw. nach dem aktuellen Element der Liste.
    \endif
  */
  enum InsertPosition
  {
    InsertHEAD,
    InsertLAST,
    InsertCURRENT
  };
  //---------------------------------------------------------------------------
  /*! \enum ElementCache
    \if english
      The pointer to the current element of a container can be stored in a buffer
      or restored from it. <br>
      This can be used, for example, to work through the list, which moves the
      current pointer, in order to then restore the current pointer.
    \endif
  
    \if german
      Der Zeiger auf das aktuelle Element eines Containers kann in einem Zwischenspeicher
      gespeichert bzw. aus diesem restauriert werden. <br>
      Damit kann z.B. die Liste durchgearbeitet werden, welches den aktuellen Zeiger
      verschiebt, und danach der aktuelle Zeiger restauriert werden.
    \endif
  */
  /*!
  \var ElementCache::M
    \if english
      The current element is copied to the buffer.
    \endif
  
    \if german
      Das aktuelles Element wird in den Zwischenspeicher kopiert.
    \endif

  \var ElementCache::RM
    \if english
      The current element is restored from the buffer.
    \endif
  
    \if german
      Das aktuelles Element wird aus dem Zwischenspeicher wieder hergestellt.
    \endif
  */
  enum ElementCache
  {
    M,
    RM
  };
  //---------------------------------------------------------------------------
  /*! \enum NodeMode
    \if english
      For NamedNode, whether subnodes insert sorted or not.
    \endif
  
    \if german
      Für NamedNode, ob diese Unterknoten sortiert aufnehmen oder nicht.
    \endif
  */
  /*!
  \var NodeMode::UNSORTED
    \if english
      Node inserts child nodes at the end, no sorting.
    \endif
  
    \if german
      Knoten fügt Unterknoten am Ende ein, keine Sortierung.
    \endif

  \var NodeMode::NAMED_SORTED
    \if english
      Node inserts subnodes sorted by name.
    \endif
  
    \if german
      Knoten fügt Unterknoten sortiert nach Bezeichnung ein.
    \endif

  \var NodeMode::ID_SORTED
    \if english
      Node inserts subnodes sorted by ID (only possible for IDNode).
    \endif
  
    \if german
      Knoten fügt Unterknoten sortiert nach ID ein.
    \endif
  */
  enum NodeMode 
  {
    UNSORTED,
    NAMED_SORTED,
    ID_SORTED
  };
  //---------------------------------------------------------------------------
  /*! \typedef int ListMode
  \if english
    Containers can handle objects differently.

    For container, whether they allow data objects more than once or only once, or
    whether they are the owner (responsible, or destroy also the data objects in
    the destructor) of these data objects. <br>
    When instantiating a list, you can also pass several modes Exclusive/Or
    connected like 'ptl::UNIQUE | ptl::RESPONSIBLE'. It is checked for bits.
  \endif

  \if german
    Container können Objekte unterschiedlich handhaben.

    Für Container, ob sie Datenobjekte mehr- oder nur einmalig zulassen oder Besitzer
    (verantwortlich, bzw. zerstören auch die Datenobjekte im Destruktor) dieser
    Datenobjekte sind. <br>
    Bei instanziierung einer Liste können auch mehrere Modie Exclusiv/Oder verbunden
    übergeben werden wie z.B. 'ptl::UNIQUE | ptl::RESPONSIBLE'. Es wird auf Bit geprüft.
  \endif

  \sa 
      - DEFAULT
      - UNIQUE
      - RESPONSIBLE;
  */
  /*!
  \var ListMode::DEFAULT
    \if english
      The list inserts pointers multiple times and is not responsible for releasing
      the memory of the referenced objects
    \endif
  
    \if german
      Die Liste fügt Zeiger auch mehrfach ein und ist nicht für die Freigabe des 
      Speichers der referenzietrten Objekte zuständig
    \endif
    \sa typedef int ListMode

  \var ListMode::UNIQUE
    \if english
      The list inserts pointers only once.
    \endif
  
    \if german
      Die Liste fügt Zeiger nur einmalig ein.
    \endif
    \sa typedef int ListMode

  \var ListMode::RESPONSIBLE
    \if english
      The list is responsible for releasing the memory of the referenced objects
      (calls the respective dtor of the objects in the list, if itself 'dying').
    \endif
  
    \if german
      Die Liste ist für die Freigabe des Speichers der referenzietrten Objekte
      zuständig (ruft den jeweiligen dtor auf, wenn selbst am 'sterben').
    \endif
    \sa typedef int ListMode
  */
  typedef int ListMode;
  const ListMode DEFAULT = 0;
  const ListMode UNIQUE = 1;
  const ListMode RESPONSIBLE = 2;
  //---------------------------------------------------------------------------
  /*! \enum SeekMode
    \if english
      For DVLists, how list items should be fetched. <br>
      SeekMode is usually passed for iteration.
    \endif
  
    \if german
      Für DVListen, wie Listenelemente geholt werden sollen. <br>
      Wird in der Regel beim Iterieren übergeben.
    \endif
  */
  enum SeekMode
  {
    //---------------------------------------------------------------------------
    /*!
      \if english
        The first element is fetched, if available.
      \endif
  
      \if german
        Es wird das erste Element geholt, wenn vorhanden.
      \endif
    */
    START,
    //---------------------------------------------------------------------------
    /*!
      \if english
        The last element is fetched, if available.
      \endif
  
      \if german
        Es wird das letzte Element geholt, wenn vorhanden.
      \endif
    */
    END,
    //---------------------------------------------------------------------------
    /*!
      \if english
        The current element is fetched, if available.
      \endif
  
      \if german
        Es wird das aktuelle Element geholt, wenn vorhanden.
      \endif
    */
    CURRENT
    //---------------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
} // namespace ptl
#endif
