#ifndef INC_PTLTDVLIST_H
#define INC_PTLTDVLIST_H
//-----------------------------------------------------------------------------
/*!
\file PtlTDVList.h
\brief
\if english
  Header for TDVList
\endif

\if german
  Header für TDVList
\endif
\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the PtlContainer module.

PtlContainer is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "PtlListTypes.h"
#include "PtlATDVList.h"
//-----------------------------------------------------------------------------
namespace ptl
{
  //---------------------------------------------------------------------------
  /* Wenn MSDev: "deletion of pointer to incomplete type 'x'; no destructor called" unterdruecken */
  #ifdef _MSC_VER
    #pragma warning( disable : 4150 )
  #endif
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    Template list class for pointers to objects.
    \details A TDVList consists of list elements, which have pointers to the
    objects to be managed. The first, last and current element can be queried.
    In addition, it is possible to select by a subsequent or previous element
    (with respect to the current element).

    An element can be inserted before or after the first, last and current element.
    By setting a #UNIQUE list, you can specify that objects may only appear once in
    the list.

    You can delete the first, last or current element, whereby the data object is
    also deleted, if this (via setting #RESPONSIBLE or #UNIQUE | #RESPONSIBLE) is
    set.

    In order to establish type security, this has been a template class with
    aggregation  on VDVList. With mostly inline methods, equivalent methods
    are stored in the VDVList called.

    \tparam <B>T_DATA</B> Type of list object pointer
  \endif

  \if german
    Template-Listenklasse für Zeiger auf Objekte.
    \details Eine TDVList besteht aus Listenelementen, welchen Zeiger auf die zu
    verwaltenden Objekte haben. Es können das erste, letzte und aktuelle Element
    erfragt werden. Zusätzlich kann nach einem nachfolgenden, bzw. vorherigen
    Element (bezüglich dem aktuellen Element) selektiert werden.

    Ein Element kann vor oder hinter dem ersten, letzten und aktuellen Element eingefügt
    werden. Dieses kann durch Einstellung einer #UNIQUE Liste in der Weise eingeschränkt
    werden, daß Objekte nur einmalig in der Liste vorkommen dürfen.

    Gelöscht werden kann das erste, letzte oder aktuelle Element, wobei das DatenObjekt
    mitgelöscht wird, wenn dieses (über Einstellung #RESPONSIBLE oder #UNIQUE | #RESPONSIBLE)
    eingestellt ist.

    Um eine Typsicherheit herzustellen, wurde dieses eine Templateklasse mit Aggregation
    auf VDVList. Mit meist inline-Methoden werden äquivalente Methoden in der VDVList
    aufgerufen.

    \tparam <B>T_DATA</B> Typ des List-Objektzeigers
  \endif

  \copyright © 1995 Helmut Jakoby
  \author Helmut Jakoby
  */
  template < class T_DATA > class TDVList : public ATDVList
  {
    public:
      //============== Konstruktoren ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor with optional parameter passing.

        The list is empty at first. With parameter eMode a list can be instantiated,
        which includes data objects several times (no parameter or #DEFAULT) or only
        once (#UNIQUE) and/or should be the owner of the data objects (#RESPONSIBLE).
        The values can be passed as OR-linked (e.g. #UNIQUE | #RESPONSIBLE).
        \param[in] eMode The list mode

        <"Example"?
      \endif
    
      \if german
        Konstruktor mit optionaler Parameterübergabe.

        Die Liste ist erst einmal leer. Durch Parameter eMode kann eine Liste instanziiert
        werden, welche Datenobjekte mehrmals (keinen Parameter oder #DEFAULT)
        oder nur einmalig (#UNIQUE) aufnimmt und/oder Besitzer der Datenobjekte
        sein soll (#RESPONSIBLE). Die Werte können Oder-Verknüpft
        (z.B. #UNIQUE | #RESPONSIBLE) übergeben werden.
        \param[in] eMode  Der Listenmodus

        <B>Beispiel</B>
      \endif
      \code{.cpp}
      TestObject * pMyTestObject = new TestObject();

      ptl::TDVList<TestObject> * pMyList = new ptl::TDVList<TestObject>();
      \endcode
      */
      TDVList( ListMode eMode = DEFAULT );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor, the list will be cleared.

        If the list was instantiated with parameter #RESPONSIBLE, all data objects
        are also destroyed.
      \endif
    
      \if german
        Destruktor, die Liste wird geleert.

        Wenn die Liste mit Parameter #RESPONSIBLE instanziiert worden sein sollte,
        werden auch alle Datenobjekte zerstört.
      \endif
      */
      virtual ~TDVList();
      //-----------------------------------------------------------------------
      /*!
      \if english
        Copy constructor.
        \param[in] rList The list whose pointers to objects are copied to this list.
        \exception eut::ErrorNException If an error occurs, a exception is thrown.
      \endif
    
      \if german
        Copy-Konstruktor.
        \param[in] rList Die Liste, deren Zeiger auf Objekte in diese Liste kopiert werden.
        \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
      \endif
      */
      TDVList( const TDVList & rList );
      //-----------------------------------------------------------------------

      //============== Methoden ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        This method inserts the pointer to a data object in the list. The current
        element is set to the new element.

        If the list was instantiated with parameter #UNIQUE and the data object is
        already in the list, it is not inserted.
        \param[in] pDataObject Is the pointer to the data object.
        \param[in] eWhere Indicates where the data object is to be inserted in the
          list. Possible parameters are #InsertHEAD, #InsertLAST and
          #InsertCURRENT .
        \param[in] eBeforeBehind Indicates whether the new element is placed before
          or after the element specified in the eWhere parameter. Possible
          parameters are #BEFORE and #BEHIND.
        \return A return value < 0 indicates an error. The following errors are
          possible: #PASSED_NO_OBJECT,  #INSTANTIATE_DVELEMENT and
          #OBJECT_ALREADY_AVAILABLE  (only at #UNIQUE).
      \endif
    
      \if german
        Diese Methode fügt den Zeiger auf ein Datenobjekt in die Liste ein. Das
        aktuelle Element wird auf das neue Element gesetzt.

        Sollte die Liste mit Parameter #UNIQUE instanziiert worden sein und das
        Datenobjekt schon in der Liste sein, wird es nicht eingefügt.
        \param[in] pDataObject Ist der Zeiger auf das Datenobjekt.
        \param[in] eWhere Zeigt an, wohin das Datenobjekt in die Liste eingefügt
          werden soll. Mögliche Parameter sind #InsertHEAD, #InsertLAST und
          #InsertCURRENT .
        \param[in] eBeforeBehind Gibt an, ob das neue Element vor oder hinter das,
          im Parameter eWhere angegebene Element, gesetzt wird. Mögliche Parameter
          sind #BEFORE und #BEHIND.
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
        Folgende Fehler sind möglich: #PASSED_NO_OBJECT,
        #INSTANTIATE_DVELEMENT und #OBJECT_ALREADY_AVAILABLE
        (nur bei #UNIQUE).
      \endif
      */
      virtual int insert( T_DATA* pDataObject,
                          InsertPosition eWhere = InsertLAST, 
                          VH eBeforeBehind = BEHIND );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns a pointer to a data object from the element position specified
        in the parameter.

        If an element was delivered, it becomes the current element, otherwise
        the current element remains what it was.
        \param[in] eElementPos From which position should the data object be
          delivered? Possible values are #FIRST_POS, #LAST_POS, #CURRENT_POS, #NEXT_POS or
          #PREVIOUS_POS.
        \return The pointer to the data object, otherwise \ref NULL_PTR "NULL_PTR".
      \endif
    
      \if german
        Liefert einen Zeiger auf ein DatenObjekt aus der im Parameter angegebenen
        Elementposition.

        Wenn ein Element geliefert wurde, wird dieses zum aktuellen Element, ansonsten
        bleibt das aktuellen Element was es war.
        \param[in] eElementPos Aus welcher Position soll das datenobjekt geliefert
          werden? Mögliche Werte sind #FIRST_POS, #LAST_POS, #CURRENT_POS, #NEXT_POS oder #PREVIOUS_POS.
        \return Der Zeiger auf das Datenobjekts, sonst \ref NULL_PTR "NULL_PTR".
      \endif
      */
      virtual T_DATA * get( ElementPosition eElementPos = FIRST_POS );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns a pointer to an object in prRetVal from the list with respect to
        the transfer parameters.

        You can use this method to iterate through the list.
        \param[in,out] prRetVal In the passed pointer a found object delivered.
        \param[in] nPosition The position relative to the search mode.
        \param[in] eMode The search mode.
        \return A return < 0 indicates an error. A return == #RANGE
          indicates the end of the list.

        <B>Parameter examples:</B>
        \li lPosition = 0; eMode = #START -> Delivers first element from list
        \li lPosition = 0; eMode = #END -> Delivers last element from list
        \li lPosition = 0; eMode = #CURRENT -> Delivers current element from list
        \li lPosition = 1; eMode = #CURRENT -> Delivers next element from list
        \li lPosition = 2; eMode = #CURRENT -> Delivers the next but one element from the list
        \li lPosition = -1; eMode = #CURRENT -> Returns previous element from list
      \endif
    
      \if german
        Liefert einen Zeiger auf ein Objekt in prRetVal aus der Liste in Bezug auf
        die Übergabe-Parameter.

        Mit dieser Methode kann durch die Liste iteriert werden.
        \param[in,out] prRetVal Im übergebenen Zeiger wird ein gefundenes Objekt
          geliefert.
        \param[in] nPosition Die Position rellativ zum Suchmodus.
        \param[in] eMode Der Suchmodus.
        \return Eine Rückgabe < 0 zeigt einen Fehler an. Eine Rückgabe == #RANGE
          zeigt das Ende der Liste an.

        <B>Parameterbeispiele:</B>
        \li lPosition =  0; eMode = #START -> Liefert erstes Element aus Liste
        \li lPosition =  0; eMode = #END -> Liefert letztes Element aus Liste
        \li lPosition =  0; eMode = #CURRENT -> Liefert aktuelles Element aus Liste
        \li lPosition =  1; eMode = #CURRENT -> Liefert nächstes Element aus Liste
        \li lPosition =  2; eMode = #CURRENT -> Liefert übernächstes Element aus Liste
        \li lPosition =  -1; eMode = #CURRENT -> Liefert vorgeriges Element aus Liste
      \endif
      */
      int get( T_DATA *& prRetVal, std::size_t nPosition, SeekMode eMode );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns a pointer to an object in prRetVal from passed position with
        respect to passed pointer to an element.

        \param[in,out] prRetVal In the passed pointer a found object is returned.
        \param[in] nPosition The position relative to the transferred pRefElement
        \param[in] pRefElement The reference element
        \return A return < 0 indicates an error. A return == #RANGE
          indicates the end of the list.
      \endif
    
      \if german
        Liefert einen Zeiger auf ein Objekt in prRetVal aus übergebener Position
        in Bezug auf übergebenen Zeiger auf ein Element.

        \param[in,out] prRetVal Im übergebenen Pointer wird ein gefundenes Objekt
          geliefert.
        \param[in] nPosition Die Position rellativ zum übergebenen pRefElement
        \param[in] pRefElement Das Referenzelement
        \return Eine Rückgabe < 0 zeigt einen Fehler an. Eine Rückgabe == #RANGE
          zeigt das Ende der Liste an.
      \endif
      */
      int get( T_DATA *& prRetVal, std::size_t nPosition, DVElement * pRefElement );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Removes the element specified as parameter from the list.

        If a subsequent element exists, it becomes the current element, otherwise
        the previous element, if it exists. If list has been instantiated with
        parameter #RESPONSIBLE, the destructor is called by the removed data
        object.
        \param[in] eElementPos Possible values are #FIRST_POS, #LAST_POS and #CURRENT_POS.
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Entfernt das Element, welches als Parameter angegeben ist, aus der Liste.

        Wenn ein nachfolgendes Element vorhanden ist, wird dieses zum aktuellen Element,
        ansonsten das vorherige Element, wenn dieses vorhanden ist. Sollte Liste mit
        Parameter #RESPONSIBLE instanziiert worden sein, wird vom entfernten
        Datenobjekte der Destruktor aufgerufen.
        \param[in] eElementPos Mögliche Werte sind #FIRST_POS, #LAST_POS und #CURRENT_POS.
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
      \endif
      */
      int remove( ElementPosition eElementPos = CURRENT_POS );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Removes all elements from the list.

        If the List has been instantiated with the #RESPONSIBLE parameter, the
        destructor is called from each removed data object.
      \endif
    
      \if german
        Entfernt alle Elemente aus Liste.

        Sollte die Liste mit Parameter #RESPONSIBLE instanziiert worden sein, wird
        von jedem entfernten Datenobjekten der Destruktor aufgerufen.
      \endif
      */
      void removeAll();
      //-----------------------------------------------------------------------
      /*!
      \if english
        If there is a PtlDV element in the list with a passed pointer, the current
        pointer is set to the PtlDV element, otherwise the current pointer remains
        what it was.
        \param[in] pDataObject The data object you are looking for
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Wenn DVElement mit übergebener Zeiger in Liste vorhanden ist, wird der
        aktuelle Zeiger auf das DVElement gesetzt, ansonsten bleibt der aktuelle
        Zeiger was er war.
        \param[in] pDataObject Das gesuchte Datenobjekt
        \return Ein Rückgabewert < 0 zeigt einen Fehler an.
      \endif
      */
      int setCurrentPosition( T_DATA* pDataObject );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the number of data objects that are equal to the passed data object.
        \param[in] pDataObject The pointer to the object
        \return The frequency of occurrence of pDataObject
      \endif
    
      \if german
        Liefert die Anzahl der Datenobjekte zurück, welche gleich des übergebenen
        Datenobjekts sind.
        \param[in] pDataObject Der Zeiger auf das Objekt
        \return Die Häufigkeit des Vorkommens von pDataObject
      \endif
      \sa AVDVList::getNumberDataObjects(...)
      */
      std::size_t getNumberDataObjects( T_DATA* pDataObject );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Returns the List mode.
        \return The ListMode which use this class. A combination of the values
        can also be returned.
      \endif
    
      \if german
        Liefert den ListModus.
        \return Der ListMode welche diese Klasse nutzen. Es kann auch eine
        Kombination der Werte geliefert werden.
      \endif
      */
      ListMode getListMode() override;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Copies all list elements of parameter rList to this list.

        You can copy data objects from subclasses, as well as subclasses can
        transfer data objects among themselves. There are not duplicates the data
        objects, but only the pointers to them.

        If there was a current element, it remains what it was, otherwise the last
        element will be to the current element.

        If the list has been instantiated with parameter #UNIQUE and a data object
        is already in the list, it is not inserted.

        \param[in] rList The list from which the elements are copied.
        \return A return value < 0 indicates an error.
      \endif
    
      \if german
        Kopiert alle Listenelemente von Parameter rList zu dieser Liste hinzu.

        Es können Datenobjekte aus Unterklassen übernommen werden, wie auch die
        Unterklassen untereinander Datenobjekte übertragen können. Es werden nicht
        die Datenobjekte, sondern nur die Zeiger auf diese dupliziert.

        Wenn es ein aktuelles Element gab, bleibt dieses was es war, ansonsten wird
        das letzte Element zum aktuellen Element.

        Sollte die Liste mit Parameter #UNIQUE instanziiert worden sein und ein
        Datenobjekt schon in der Liste sein, wird es nicht eingefügt.

        \param[in] rList Die Liste, aus der die Elemente übernommen werden.
        \return Ein Rückgabewerte < 0 zeigt einen Fehler an.
      \endif
      */
      virtual int addElementFrom( TDVList& rList );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Assignment operator=

        When an assignment is made, all elements are removed from this list (if the
        list was instantiated with the parameter #RESPONSIBLE, all data objects are
        also destroyed) and then all data objects from the assigning list are
        transferred to this list.
        \param[in] rList The list to be assigned.

       It should be called getErrorCode() after this call.
      \endif
    
      \if german
        Zuweisungsoperator=

        Bei einer Zuweisung werden alle Elemente aus dieser Liste entfernt (wenn die
        Liste mit Parameter #RESPONSIBLE instanziiert worden sein sollte, werden auch
        alle Datenobjekte zerstört.) und dann alle Datenobjekte aus der zuweisenden
        Liste in diese Liste übernommen.
        \param[in] rList Die zuzuweisende Liste.

        Es sollte nach diesem Aufruf getErrorCode() aufgerufen werden.
      \endif
      */
      TDVList<T_DATA> & operator= ( const TDVList& rList );
      //-----------------------------------------------------------------------
  };

  //---------------------------------------------------------------------------
  template <  class T_DATA  >
  TDVList< T_DATA >::TDVList (ListMode eMode)
  : ATDVList(eMode)
  {
  }
  //---------------------------------------------------------------------------
  template <  class T_DATA  >
  TDVList< T_DATA >::~TDVList ()
  {
    removeAll();
  }
  //---------------------------------------------------------------------------
  template <  class T_DATA  >
  TDVList< T_DATA >::TDVList (const TDVList & rList)
  {
    operator=( rList );
  }
  //---------------------------------------------------------------------------
  template <  class T_DATA  >
  T_DATA * TDVList< T_DATA >::get (ElementPosition eElementPos)
  {
    return reinterpret_cast<T_DATA*>(m_VDVListe->get(eElementPos));
  }
  //---------------------------------------------------------------------------
  template <  class T_DATA  >
  int TDVList< T_DATA >::get (T_DATA *& prRetVal, std::size_t nPosition, SeekMode eMode)
  {
    int      t_iErr = 0;
    void  *  t_pTempData = 0;
  
    t_iErr = m_VDVListe->get(t_pTempData, nPosition, eMode);
    prRetVal = reinterpret_cast<T_DATA*>(t_pTempData);
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  template <  class T_DATA  >
  int TDVList< T_DATA >::get (T_DATA *& prRetVal, std::size_t nPosition, DVElement * pRefElement)
  {
    int      t_iErr = 0;
    void  *  t_pTempData = 0;
  
    t_iErr = m_VDVListe->get(prRetVal, nPosition, pRefElement);
    prRetVal = reinterpret_cast<T_DATA*>(t_pTempData);
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  template <  class T_DATA  >
  int TDVList< T_DATA >::insert (T_DATA* pDataObject, InsertPosition eWhere, VH eBeforeBehind)
  {
    return m_VDVListe->insert(pDataObject, eWhere, eBeforeBehind);
  }
  //---------------------------------------------------------------------------
  template <  class T_DATA  >
  int TDVList< T_DATA >::remove (ElementPosition eElementPos)
  {
    int       t_iErr = 0;
    T_DATA  * t_pDelData = 0;
  
    // Datenobjekt muß evtl gelöscht werden, also holen; wird aktuell
    t_pDelData = reinterpret_cast<T_DATA*>(m_VDVListe->get(eElementPos));
    if (!t_pDelData)
    {
      t_iErr = OBJEKT_UNAVAILABLE;
    }
    else
    {
      // Element (jetzt aktuell) aus Liste entfernen
      t_iErr = m_VDVListe->remove(CURRENT_POS);
    
      // Wenn RESPONSIBLE-Bit gesetzt und Datenobjekt da, dann auch Datenobjekte löschen.
      if ((m_VDVListe->getListMode() & RESPONSIBLE) && (t_pDelData))
      {
        // Wenn Datenobjekt nicht nocheinmal in Liste vorhanden, dann löschen
        if (getNumberDataObjects(t_pDelData) == 0)
        {
          //delete static_cast<T_DATA*>(t_pDelData);
          delete t_pDelData;
        }
      }
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  template <  class T_DATA  >
  void TDVList< T_DATA >::removeAll ()
  {
    // Solange Elemente in Liste; entfernen
    while ( this->get( FIRST_POS) )
    {
      this->remove( FIRST_POS );
    } // Ende while
  }
  //---------------------------------------------------------------------------
  template <  class T_DATA  >
  int TDVList< T_DATA >::setCurrentPosition (T_DATA* pDataObject)
  {
    return m_VDVListe->setCurrentPosition(pDataObject);
  }
  //---------------------------------------------------------------------------
  template <  class T_DATA  >
  size_t TDVList< T_DATA >::getNumberDataObjects (T_DATA* pDataObject)
  {
    return m_VDVListe->getNumberDataObjects(pDataObject);
  }
  //---------------------------------------------------------------------------
  template <  class T_DATA  >
  ListMode TDVList< T_DATA >::getListMode ()
  {
    return m_VDVListe->getListMode();
  }
  //---------------------------------------------------------------------------
  template <  class T_DATA  >
  int TDVList< T_DATA >::addElementFrom (TDVList& rList)
  {
    return m_VDVListe->addElementFrom(*rList.m_VDVListe);
  }
  //---------------------------------------------------------------------------
  template <  class T_DATA  >
  TDVList<T_DATA> & TDVList< T_DATA >::operator= (const TDVList& rList)
  {
    // Wenn RESPONSIBLE-Bit gesetzt, dann evtl auch Datenobjekte löschen.
    if (m_VDVListe->getListMode() & RESPONSIBLE)
    {
      this->removeAll();
    }
    // Hier braucht nicht auf "if (this != &rList)..." abgefragt werden, weil
    // in m_VDVListe (Klasse VDVList) auf Gleichheit von m_VDVListe und
    // übergebenen rList.m_VDVListe verglichen wird und bei Gleichheit keine
    // Übertragung der DatenObjekte vorgenommen wird.
    // ACHTUNG! Damit hier keine Mißverständnise auftreten. Es wird NICHT der
    // Pointer von rList.m_VDVListe nach m_VDVListe kopiert. In der Klasse
    // aus m_VDVListe bzw. rList.m_VDVListe ist der operator= überladen!!!!!
    // Es werden alle Datenobjekt-Pointer aus rList.m_VDVListe in m_VDVListe in
    // neue DVElemente übertragen.
    * m_VDVListe = * rList.m_VDVListe;
    return * this;
  }

  //---------------------------------------------------------------------------
  /* Wenn MSDev, "deletion of pointer to incomplete type 'x'; no destructor called" freigeben */
  #ifdef _MSC_VER
    #pragma warning( default : 4150 )
  #endif
  //---------------------------------------------------------------------------
} // namespace ptl
#endif
